@@ -19,6 +19,7 @@ pub use self::keys::Keyboard;
 pub use self::logs::Logs;
 pub use self::notification::Notifications;
 pub use self::pane::Pane;
+pub use self::platform_specific::PlatformSpecific;
 pub use self::preview::Preview;
 pub use self::proxy::Proxy;
 pub use self::server::Server;
@@ -26,7 +27,7 @@ pub use self::sidebar::Sidebar;
 use crate::appearance::theme::Styles;
 use crate::appearance::{self, Appearance};
 use crate::audio::{self};
-use crate::environment::config_dir;
+use crate::serde::deserialize_positive_integer_maybe;
 use crate::server::{ConfigMap as ServerMap, ServerName};
 use crate::{Theme, environment};
 
@@ -39,6 +40,7 @@ pub mod keys;
 pub mod logs;
 pub mod notification;
 pub mod pane;
+pub mod platform_specific;
 pub mod preview;
 pub mod proxy;
 pub mod server;
@@ -66,6 +68,7 @@ pub struct Config {
     pub actions: Actions,
     pub ctcp: Ctcp,
     pub logs: Logs,
+    pub platform_specific: PlatformSpecific,
 }
 
 #[derive(Debug, Clone, Copy, Deserialize)]
@@ -111,7 +114,7 @@ impl Default for Scrollbar {
 #[serde(default)]
 pub struct Font {
     pub family: Option<String>,
-    #[serde(deserialize_with = "deserialize_optional_font_size")]
+    #[serde(deserialize_with = "deserialize_positive_integer_maybe")]
     pub size: Option<u8>,
     #[serde(deserialize_with = "deserialize_font_weight_from_string")]
     pub weight: font::Weight,
@@ -131,26 +134,6 @@ impl Default for Font {
     }
 }
 
-pub fn deserialize_optional_font_size<'de, D>(
-    deserializer: D,
-) -> Result<Option<u8>, D::Error>
-where
-    D: Deserializer<'de>,
-{
-    let integer_maybe: Option<u8> = Deserialize::deserialize(deserializer)?;
-
-    if let Some(integer) = integer_maybe
-        && integer == 0
-    {
-        Err(serde::de::Error::invalid_value(
-            serde::de::Unexpected::Unsigned(integer.into()),
-            &"any positive integer",
-        ))
-    } else {
-        Ok(integer_maybe)
-    }
-}
-
 fn deserialize_font_weight_from_string<'de, D>(
     deserializer: D,
 ) -> Result<font::Weight, D::Error>
@@ -333,6 +316,7 @@ impl Config {
             pub actions: Actions,
             pub ctcp: Ctcp,
             pub logs: Logs,
+            pub platform_specific: PlatformSpecific,
         }
 
         impl Default for Configuration {
@@ -355,15 +339,14 @@ impl Config {
                     actions: Actions::default(),
                     ctcp: Ctcp::default(),
                     logs: Logs::default(),
+                    platform_specific: PlatformSpecific::default(),
                 }
             }
         }
 
         let path = Self::path();
         if !path.try_exists()? {
-            return Err(Error::ConfigMissing {
-                has_yaml_config: has_yaml_config()?,
-            });
+            return Err(Error::ConfigMissing);
         }
         let content = fs::read_to_string(path)
             .await
@@ -389,6 +372,7 @@ impl Config {
             actions,
             ctcp,
             logs,
+            platform_specific,
         } = serde_ignored::deserialize(config, |ignored| {
             log::warn!("[config.toml] Ignoring unknown setting: {ignored}");
         })
@@ -418,6 +402,7 @@ impl Config {
             actions,
             ctcp,
             logs,
+            platform_specific,
         })
     }
 
@@ -546,11 +531,6 @@ pub fn random_nickname_with_seed<R: Rng>(rng: &mut R) -> String {
     rand_nick
 }
 
-/// Has YAML configuration file.
-fn has_yaml_config() -> Result<bool, Error> {
-    Ok(config_dir().join("config.yaml").try_exists()?)
-}
-
 #[derive(Debug, Error, Clone)]
 pub enum Error {
     #[error("config could not be read: {0}")]
@@ -580,7 +560,7 @@ pub enum Error {
     )]
     DuplicateSaslPassword,
     #[error("Config does not exist")]
-    ConfigMissing { has_yaml_config: bool },
+    ConfigMissing,
 }
 
 impl From<std::io::Error> for Error {
