@@ -1,7 +1,11 @@
+use std::collections::HashSet;
+
 use chrono::{DateTime, Local, Utc};
+use iced::Color;
 use serde::{Deserialize, Deserializer};
 
 pub use self::channel::Channel;
+use crate::appearance::theme::{alpha_color, alpha_color_calculate};
 use crate::config::buffer::nickname::Nickname;
 
 pub mod channel;
@@ -31,6 +35,7 @@ pub struct Buffer {
     pub mark_as_read: MarkAsRead,
     pub url: Url,
     pub line_spacing: u32,
+    pub scroll_position_on_open: ScrollPosition,
 }
 
 #[derive(Debug, Clone, Deserialize, Default)]
@@ -67,6 +72,14 @@ pub struct Url {
     pub prompt_before_open: bool,
 }
 
+#[derive(Debug, Default, Clone, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum ScrollPosition {
+    #[default]
+    OldestUnread,
+    Newest,
+}
+
 #[derive(Debug, Clone, Deserialize)]
 #[serde(default)]
 pub struct MarkAsRead {
@@ -117,23 +130,47 @@ pub enum OnBufferCloseCondition {
     ScrolledToBottom,
 }
 
+#[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
+pub struct SysInfo {
+    pub cpu: bool,
+    pub memory: bool,
+    pub gpu: bool,
+    pub os: bool,
+    pub uptime: bool,
+}
+
+impl Default for SysInfo {
+    fn default() -> Self {
+        Self {
+            cpu: true,
+            memory: true,
+            gpu: true,
+            os: true,
+            uptime: true,
+        }
+    }
+}
+
 #[derive(Debug, Clone, Deserialize)]
 #[serde(default)]
 pub struct Commands {
     pub show_description: bool,
+    pub sysinfo: SysInfo,
 }
 
 impl Default for Commands {
     fn default() -> Self {
         Self {
             show_description: true,
+            sysinfo: SysInfo::default(),
         }
     }
 }
 
 #[derive(Debug, Clone, Copy)]
 pub enum Away {
-    Dimmed(Option<f32>),
+    Dimmed(Dimmed),
     None,
 }
 
@@ -145,7 +182,7 @@ impl Away {
 
 impl Default for Away {
     fn default() -> Self {
-        Away::Dimmed(None)
+        Away::Dimmed(Dimmed::default())
     }
 }
 
@@ -169,13 +206,13 @@ impl<'de> Deserialize<'de> for Away {
         let repr = AppearanceRepr::deserialize(deserializer)?;
         match repr {
             AppearanceRepr::String(s) => match s.as_str() {
-                "dimmed" => Ok(Away::Dimmed(None)),
+                "dimmed" => Ok(Away::Dimmed(Dimmed(None))),
                 "solid" | "none" => Ok(Away::None),
                 _ => Err(serde::de::Error::custom(format!(
                     "unknown appearance: {s}",
                 ))),
             },
-            AppearanceRepr::Struct(s) => Ok(Away::Dimmed(s.dimmed)),
+            AppearanceRepr::Struct(s) => Ok(Away::Dimmed(Dimmed(s.dimmed))),
         }
     }
 }
@@ -183,6 +220,7 @@ impl<'de> Deserialize<'de> for Away {
 #[derive(Debug, Clone, Default, Deserialize)]
 #[serde(default)]
 pub struct ServerMessages {
+    pub condense: Condensation,
     pub topic: ServerMessage,
     pub join: ServerMessage,
     pub part: ServerMessage,
@@ -230,6 +268,81 @@ impl ServerMessages {
             source::server::Kind::ChangeTopic => Some(&self.change_topic),
         }
     }
+
+    pub fn dimmed(&self, server: Option<&source::Server>) -> Option<&Dimmed> {
+        server.and_then(|server| {
+            self.get(server).and_then(|kind| kind.dimmed.as_ref())
+        })
+    }
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
+pub struct Condensation {
+    pub messages: HashSet<CondensationMessage>,
+    pub format: CondensationFormat,
+    #[serde(deserialize_with = "deserialize_dimmed_maybe")]
+    pub dimmed: Option<Dimmed>,
+}
+
+impl Default for Condensation {
+    fn default() -> Self {
+        Self {
+            messages: HashSet::new(),
+            format: CondensationFormat::default(),
+            dimmed: Some(Dimmed(None)),
+        }
+    }
+}
+
+impl Condensation {
+    pub fn kind(&self, server: &source::Server) -> bool {
+        match server.kind() {
+            source::server::Kind::Join => {
+                self.messages.contains(&CondensationMessage::Join)
+            }
+            source::server::Kind::Part => {
+                self.messages.contains(&CondensationMessage::Part)
+            }
+            source::server::Kind::Quit => {
+                self.messages.contains(&CondensationMessage::Quit)
+            }
+            source::server::Kind::ChangeNick => {
+                self.messages.contains(&CondensationMessage::ChangeNick)
+            }
+            source::server::Kind::ReplyTopic
+            | source::server::Kind::ChangeHost
+            | source::server::Kind::ChangeMode
+            | source::server::Kind::MonitoredOnline
+            | source::server::Kind::MonitoredOffline
+            | source::server::Kind::StandardReply(_)
+            | source::server::Kind::WAllOps
+            | source::server::Kind::Kick
+            | source::server::Kind::ChangeTopic => false,
+        }
+    }
+
+    pub fn any(&self) -> bool {
+        !self.messages.is_empty()
+    }
+}
+
+#[derive(Debug, Clone, Deserialize, Eq, Hash, PartialEq)]
+#[serde(rename_all = "kebab-case")]
+pub enum CondensationMessage {
+    Join,
+    Part,
+    Quit,
+    ChangeNick,
+}
+
+#[derive(Debug, Default, Clone, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum CondensationFormat {
+    #[default]
+    Brief,
+    Detailed,
+    Full,
 }
 
 #[derive(Debug, Clone, Deserialize)]
@@ -240,6 +353,8 @@ pub struct ServerMessage {
     pub username_format: UsernameFormat,
     pub exclude: Vec<String>,
     pub include: Vec<String>,
+    #[serde(deserialize_with = "deserialize_dimmed_maybe")]
+    pub dimmed: Option<Dimmed>,
 }
 
 impl Default for ServerMessage {
@@ -250,6 +365,7 @@ impl Default for ServerMessage {
             username_format: UsernameFormat::default(),
             exclude: Vec::default(),
             include: Vec::default(),
+            dimmed: Some(Dimmed(None)),
         }
     }
 }
@@ -354,13 +470,81 @@ impl Buffer {
             return None;
         }
 
-        Some(format!(
-            "{} ",
+        Some(
             self.timestamp.brackets.format(
                 date_time
                     .with_timezone(&Local)
-                    .format(&self.timestamp.format)
-            )
+                    .format(&self.timestamp.format),
+            ),
+        )
+    }
+
+    pub fn format_range_timestamp(
+        &self,
+        start_date_time: &DateTime<Utc>,
+        end_date_time: &DateTime<Utc>,
+    ) -> Option<(String, String, String)> {
+        if self.timestamp.format.is_empty() {
+            return None;
+        }
+
+        // Since timestamp ranges are used without a nickname or message marker,
+        // do not include space delimiter in the format.
+        Some((
+            self.timestamp
+                .brackets
+                .format(format!(
+                    "{}",
+                    start_date_time
+                        .with_timezone(&Local)
+                        .format(&self.timestamp.format)
+                ))
+                .to_string(),
+            " \u{2013} ".to_string(),
+            self.timestamp
+                .brackets
+                .format(format!(
+                    "{}",
+                    end_date_time
+                        .with_timezone(&Local)
+                        .format(&self.timestamp.format)
+                ))
+                .to_string(),
         ))
     }
 }
+
+#[derive(Debug, Default, Clone, Copy, Deserialize)]
+pub struct Dimmed(Option<f32>);
+
+impl Dimmed {
+    pub fn transform_color(&self, color: Color, background: Color) -> Color {
+        match self.0 {
+            // Calculate alpha based on background and foreground.
+            None => alpha_color_calculate(0.20, 0.61, background, color),
+            // Calculate alpha based on user defined alpha value.
+            Some(a) => alpha_color(color, a),
+        }
+    }
+}
+
+pub fn deserialize_dimmed_maybe<'de, D>(
+    deserializer: D,
+) -> Result<Option<Dimmed>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    #[derive(Deserialize)]
+    #[serde(untagged)]
+    enum Data {
+        Boolean(bool),
+        Float(f32),
+    }
+
+    let dimmed_maybe: Option<Data> = Deserialize::deserialize(deserializer)?;
+
+    Ok(dimmed_maybe.and_then(|dimmed| match dimmed {
+        Data::Boolean(dim) => dim.then_some(Dimmed(None)),
+        Data::Float(dim) => Some(Dimmed(Some(dim))),
+    }))
+}
