@@ -92,11 +92,13 @@ pub struct Server {
     /// Enable WHO polling. Defaults to `true`.
     pub who_poll_enabled: bool,
     /// WHO poll interval for servers without away-notify.
-    #[serde(deserialize_with = "deserialize_duration_from_u64")]
+    #[serde(deserialize_with = "deserialize_who_poll_interval")]
     pub who_poll_interval: Duration,
     /// A list of nicknames to monitor (if MONITOR is supported by the server).
     pub monitor: Vec<String>,
     pub chathistory: bool,
+    #[serde(deserialize_with = "deserialize_anti_flood")]
+    pub anti_flood: Duration,
     #[serde(skip)]
     pub order: u16,
 }
@@ -209,6 +211,7 @@ impl Default for Server {
             who_poll_interval: Duration::from_secs(2),
             monitor: Vec::default(),
             chathistory: true,
+            anti_flood: Duration::from_millis(2000),
             order: 0,
         }
     }
@@ -332,12 +335,36 @@ pub struct Filters {
     pub ignore: Vec<String>,
 }
 
-fn deserialize_duration_from_u64<'de, D>(
+fn deserialize_anti_flood<'de, D>(deserializer: D) -> Result<Duration, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let milliseconds: u64 = Deserialize::deserialize(deserializer)?;
+
+    if !(100..=60000).contains(&milliseconds) {
+        Err(serde::de::Error::invalid_value(
+            serde::de::Unexpected::Unsigned(milliseconds),
+            &"integer in the range 100 .. 60000",
+        ))
+    } else {
+        Ok(Duration::from_millis(milliseconds))
+    }
+}
+
+fn deserialize_who_poll_interval<'de, D>(
     deserializer: D,
 ) -> Result<Duration, D::Error>
 where
     D: Deserializer<'de>,
 {
     let seconds: u64 = Deserialize::deserialize(deserializer)?;
-    Ok(Duration::from_secs(seconds.clamp(1, 3600)))
+
+    if !(1..=3600).contains(&seconds) {
+        Err(serde::de::Error::invalid_value(
+            serde::de::Unexpected::Unsigned(seconds),
+            &"integer in the range 1 .. 3600",
+        ))
+    } else {
+        Ok(Duration::from_secs(seconds))
+    }
 }
