@@ -1,8 +1,9 @@
 use std::collections::{HashMap, HashSet, hash_map};
 
-use chrono::{DateTime, Utc};
+use chrono::{DateTime, Local, NaiveDate, Utc};
 use futures::future::BoxFuture;
 use futures::{Future, FutureExt, future};
+use itertools::Itertools;
 use tokio::time::Instant;
 
 use super::filter::{Filter, FilterChain};
@@ -67,9 +68,10 @@ impl Manager {
     pub fn clear_messages(
         &mut self,
         kind: history::Kind,
+        clients: &client::Map,
     ) -> Option<BoxFuture<'static, Message>> {
         if let Some(history) = self.data.map.get_mut(&kind) {
-            let task = history.flush(None);
+            let task = history.flush(None, clients.get_seed(&kind));
 
             match history {
                 History::Full {
@@ -97,13 +99,17 @@ impl Manager {
     pub fn track(
         &mut self,
         new_resources: HashSet<Resource>,
+        clients: Option<&client::Map>,
     ) -> Vec<BoxFuture<'static, Message>> {
         let added = new_resources.difference(&self.resources).cloned();
         let removed = self.resources.difference(&new_resources).cloned();
 
         let added = added.into_iter().map(|resource| {
+            let seed =
+                clients.and_then(|clients| clients.get_seed(&resource.kind));
+
             async move {
-                history::load(resource.kind.clone())
+                history::load(resource.kind.clone(), seed)
                     .map(move |result| Message::LoadFull(resource.kind, result))
                     .await
             }
@@ -136,7 +142,7 @@ impl Manager {
                 self.data.load_full(kind.clone(), loaded);
                 log::debug!("loaded history for {kind}: {len} messages");
 
-                self.block_messages(kind.clone(), clients, buffer_config);
+                self.process_messages(kind.clone(), clients, buffer_config);
 
                 return Some(Event::Loaded(kind));
             }
@@ -209,25 +215,40 @@ impl Manager {
         &mut self.filters
     }
 
-    pub fn tick(&mut self, now: Instant) -> Vec<BoxFuture<'static, Message>> {
-        self.data.flush_all(now)
+    pub fn tick(
+        &mut self,
+        now: Instant,
+        clients: &client::Map,
+    ) -> Vec<BoxFuture<'static, Message>> {
+        self.data.flush_all(now, clients)
     }
 
     pub fn close(
         &mut self,
         kind: history::Kind,
+        clients: &client::Map,
     ) -> Option<impl Future<Output = Message> + use<>> {
         let history = self.data.map.remove(&kind)?;
 
-        Some(history.close().map(|result| Message::Closed(kind, result)))
+        Some(
+            history
+                .close(clients.get_seed(&kind))
+                .map(|result| Message::Closed(kind, result)),
+        )
     }
 
-    pub fn exit(&mut self) -> impl Future<Output = Message> + use<> {
+    pub fn exit(
+        &mut self,
+        clients: &client::Map,
+    ) -> impl Future<Output = Message> + use<> {
         let map = std::mem::take(&mut self.data).map;
+        let seeds: Vec<Option<history::Seed>> =
+            map.keys().map(|kind| clients.get_seed(kind)).collect();
+        let seeded_map = map.into_iter().zip(seeds);
 
         async move {
-            let tasks = map.into_iter().map(|(kind, state)| {
-                state.close().map(move |result| (kind, result))
+            let tasks = seeded_map.into_iter().map(|((kind, state), seed)| {
+                state.close(seed).map(move |result| (kind, result))
             });
 
             Message::Exited(future::join_all(tasks).await)
@@ -319,7 +340,22 @@ impl Manager {
                     buffer_config,
                 );
 
-                self.data.add_message(kind, message)
+                let condensers = (message
+                    .can_condense(&buffer_config.server_messages.condense)
+                    && !message.blocked)
+                    .then_some((kind.clone(), message.clone()));
+
+                let future = self.data.add_message(kind, message);
+
+                if let Some((kind, message)) = condensers {
+                    self.condense_message(
+                        message,
+                        &kind,
+                        &buffer_config.server_messages.condense,
+                    );
+                }
+
+                future
             },
         )
     }
@@ -614,12 +650,96 @@ impl Manager {
             .filter_message_of_kind(message, kind);
     }
 
-    pub fn block_messages(
+    // Whether the message can & should be condensed should be determined prior
+    // to calling this function
+    pub fn condense_message(
+        &mut self,
+        message: crate::Message,
+        kind: &history::Kind,
+        config: &config::buffer::Condensation,
+    ) {
+        if let Some(History::Full { messages, .. }) =
+            self.data.map.get_mut(kind)
+        {
+            let fuzz_seconds = chrono::Duration::seconds(1);
+
+            let start = message.server_time - fuzz_seconds;
+            let end = message.server_time + fuzz_seconds;
+
+            let start_index = match messages
+                .binary_search_by(|stored| stored.server_time.cmp(&start))
+            {
+                Ok(match_index) => match_index,
+                Err(sorted_insert_index) => sorted_insert_index,
+            };
+            let end_index = match messages
+                .binary_search_by(|stored| stored.server_time.cmp(&end))
+            {
+                Ok(match_index) => match_index,
+                Err(sorted_insert_index) => sorted_insert_index,
+            };
+
+            if let Some(insert_position) = messages[start_index..end_index]
+                .iter()
+                .position(|stored| stored.hash == message.hash)
+                .map(|position| position + start_index)
+            {
+                let start = messages
+                    .iter()
+                    .take(insert_position)
+                    .rev()
+                    .position(|message| {
+                        !message.blocked && !message.can_condense(config)
+                    })
+                    .map_or(0, |position| insert_position - position);
+
+                let end = messages
+                    .iter()
+                    .skip(insert_position)
+                    .position(|message| {
+                        !message.blocked && !message.can_condense(config)
+                    })
+                    .map_or(messages.len(), |position| {
+                        insert_position + position
+                    });
+
+                let mut condensable_messages = messages[start..end]
+                    .iter_mut()
+                    .collect::<Vec<&mut message::Message>>(
+                );
+
+                let condensed_message = message::condense(
+                    &condensable_messages
+                        .iter()
+                        .map(|message| &**message)
+                        .collect::<Vec<&message::Message>>(),
+                    config,
+                );
+
+                condensable_messages
+                    .iter_mut()
+                    .for_each(|message| message.condensed = None);
+
+                if let Some(first_message) = condensable_messages.first_mut() {
+                    first_message.condensed = condensed_message;
+                }
+            }
+        }
+    }
+
+    // Block and condense messages
+    pub fn process_messages(
         &mut self,
         kind: history::Kind,
         clients: &client::Map,
         buffer_config: &config::Buffer,
     ) {
+        #[derive(PartialEq)]
+        enum CondensationKey {
+            Condensable(NaiveDate),
+            Singular,
+        }
+
         if let Some(history) = self.data.map.get_mut(&kind) {
             let messages = match history {
                 History::Full { messages, .. } => messages,
@@ -637,8 +757,10 @@ impl Manager {
                             buffer_config.server_messages.get(source)
                         {
                             // Check if target is a channel, and if included/excluded.
-                            if let message::Target::Channel { channel, .. } =
-                                &message.target
+                            if let message::Target::Channel { channel, .. }
+                            | message::Target::Highlights {
+                                channel, ..
+                            } = &message.target
                                 && !server_message
                                     .should_send_message(channel.as_str())
                             {
@@ -647,10 +769,18 @@ impl Manager {
                                 let nick = match source.nick() {
                                     Some(nick) => Some(nick.clone()),
                                     None => message.plain().and_then(|s| {
+                                        let server = if let Some(server) = kind.server() {
+                                            Some(server)
+                                        } else if let message::Target::Highlights { server, .. } =
+                                            &message.target
+                                        {
+                                            Some(server)
+                                        } else {
+                                            None
+                                        };
+
                                         let casemapping = clients
-                                            .get_casemapping_or_default(
-                                                kind.server(),
-                                            );
+                                            .get_casemapping_or_default(server);
 
                                         s.split(' ').nth(1).map(|nick| {
                                             Nick::from_str(nick, casemapping)
@@ -674,35 +804,99 @@ impl Manager {
                             message.server_time,
                         );
                     }
+                    message::Source::Internal(
+                        message::source::Internal::Status(status),
+                    ) => {
+                        if let Some(internal_message) =
+                            buffer_config.internal_messages.get(status)
+                        {
+                            if !internal_message.enabled {
+                                message.blocked = true;
+                            } else if let Some(seconds) = internal_message.smart
+                            {
+                                message.blocked = smart_filter_internal_message(
+                                    message, &seconds,
+                                );
+                            }
+                        }
+                    }
                     _ => (),
                 }
             });
 
             let chain = FilterChain::borrow(&self.filters);
 
-            match &kind {
-                history::Kind::Highlights => {
-                    messages.iter_mut().for_each(|message| {
-                        if message.blocked {
-                            return;
-                        }
-
-                        chain.filter_message_of_kind(message, &kind);
-                    });
+            messages.iter_mut().for_each(|message| {
+                if message.blocked {
+                    return;
                 }
-                _ => {
-                    messages.iter_mut().for_each(|message| {
-                        if message.blocked {
-                            return;
-                        }
 
-                        chain.filter_message_of_kind(message, &kind);
-                    });
-                }
-            }
+                chain.filter_message_of_kind(message, &kind);
+            });
+
+            messages
+                .iter_mut()
+                .filter(|message| !message.blocked)
+                .chunk_by(|message| {
+                    if message
+                        .can_condense(&buffer_config.server_messages.condense)
+                    {
+                        CondensationKey::Condensable(
+                            message
+                                .server_time
+                                .with_timezone(&Local)
+                                .date_naive(),
+                        )
+                    } else {
+                        CondensationKey::Singular
+                    }
+                })
+                .into_iter()
+                .for_each(|(key, chunk)| match key {
+                    CondensationKey::Condensable(_) => {
+                        let mut condensable_messages =
+                            chunk.collect::<Vec<&mut message::Message>>();
+
+                        let condensed_message = message::condense(
+                            &condensable_messages
+                                .iter()
+                                .map(|message| &**message)
+                                .collect::<Vec<&message::Message>>(),
+                            &buffer_config.server_messages.condense,
+                        );
+
+                        condensable_messages
+                            .iter_mut()
+                            .for_each(|message| message.condensed = None);
+
+                        if let Some(first_message) =
+                            condensable_messages.first_mut()
+                        {
+                            first_message.condensed = condensed_message;
+                        }
+                    }
+                    CondensationKey::Singular => (),
+                });
         }
 
-        log::debug!("blocked messages in {kind}");
+        log::debug!("processed messages in {kind}");
+    }
+
+    pub fn renormalize_messages(
+        &mut self,
+        kind: &history::Kind,
+        clients: &client::Map,
+    ) {
+        if let Some(history) = self.data.map.get_mut(kind)
+            && let Some(seed) = clients.get_seed(kind)
+        {
+            let messages = match history {
+                History::Full { messages, .. } => messages,
+                History::Partial { messages, .. } => messages,
+            };
+
+            history::renormalize_messages(messages, seed);
+        }
     }
 }
 
@@ -822,13 +1016,15 @@ impl Data {
             return None;
         };
 
-        // Perform filtering that depends on the current time.
-
-        let filtered = messages
+        let processed = messages
             .iter()
-            .filter(|message| {
+            .flat_map(|message| {
                 if message.blocked {
-                    false
+                    None
+                } else if message
+                    .can_condense(&buffer_config.server_messages.condense)
+                {
+                    message.condensed.as_ref().map(std::convert::AsRef::as_ref)
                 } else {
                     match message.target.source() {
                         message::Source::Internal(
@@ -838,31 +1034,32 @@ impl Data {
                                 buffer_config.internal_messages.get(status)
                             {
                                 if !internal_message.enabled {
-                                    return false;
-                                }
-
-                                if let Some(seconds) = internal_message.smart {
-                                    return !smart_filter_internal_message(
+                                    return None;
+                                } else if let Some(seconds) =
+                                    internal_message.smart
+                                {
+                                    return (!smart_filter_internal_message(
                                         message, &seconds,
-                                    );
+                                    ))
+                                    .then_some(message);
                                 }
                             }
 
-                            true
+                            Some(message)
                         }
-                        _ => true,
+                        _ => Some(message),
                     }
                 }
             })
             .collect::<Vec<_>>();
 
-        let total = filtered.len();
+        let total = processed.len();
         let with_access_levels = buffer_config.nickname.show_access_levels;
         let truncate = buffer_config.nickname.truncate;
 
         let max_nick_chars =
             buffer_config.nickname.alignment.is_right().then(|| {
-                filtered
+                processed
                     .iter()
                     .filter_map(|message| {
                         if let message::Source::User(user) =
@@ -892,7 +1089,7 @@ impl Data {
         let max_prefix_chars =
             buffer_config.nickname.alignment.is_right().then(|| {
                 if matches!(kind, history::Kind::Channel(..)) {
-                    filtered
+                    processed
                         .iter()
                         .filter_map(|message| {
                             message.target.prefixes().map(|prefixes| {
@@ -912,18 +1109,55 @@ impl Data {
                 }
             });
 
-        let has_read_messages = read_marker
-            .map(|marker| {
-                filtered
-                    .iter()
-                    .any(|message| message.server_time <= marker.date_time())
-            })
-            .unwrap_or_default();
+        // The right-aligned nicknames setting expects timestamps to have a
+        // constant character count to function, so we can utilize that
+        // expectation in this calculation
+        let max_excess_timestamp_chars = (buffer_config
+            .nickname
+            .alignment
+            .is_right()
+            && buffer_config.server_messages.condense.any())
+        .then(|| {
+            processed
+                .iter()
+                .find_map(|message| {
+                    if let message::Source::Internal(
+                        message::source::Internal::Condensed(end_server_time),
+                    ) = message.target.source()
+                        && message.server_time != *end_server_time
+                    {
+                        Some(
+                            buffer_config
+                                .format_range_timestamp(
+                                    &message.server_time,
+                                    end_server_time,
+                                )
+                                .map(
+                                    |(start_timestamp, dash, end_timestamp)| {
+                                        start_timestamp.chars().count()
+                                            + dash.chars().count()
+                                            + end_timestamp.chars().count()
+                                    },
+                                )
+                                .unwrap_or_default()
+                                - buffer_config
+                                    .format_timestamp(&message.server_time)
+                                    .map(|timestamp| {
+                                        timestamp.chars().count() + 1
+                                    })
+                                    .unwrap_or_default(),
+                        )
+                    } else {
+                        None
+                    }
+                })
+                .unwrap_or_default()
+        });
 
-        let first_without_limit = filtered.first().copied();
-        let last_without_limit = filtered.last().copied();
+        let first_without_limit = processed.first().copied();
+        let last_without_limit = processed.last().copied();
 
-        let limited = with_limit(limit, filtered.into_iter());
+        let limited = with_limit(limit, processed.into_iter());
 
         let first_with_limit = limited.first();
         let last_with_limit = limited.last();
@@ -936,10 +1170,7 @@ impl Data {
                     message.server_time <= read_marker.date_time()
                 })
                 .map_or_else(
-                    || {
-                        // Backlog is before this limit view of messages
-                        if has_read_messages { 0 } else { limited.len() }
-                    },
+                    || 0, // Backlog is before this limit view of messages
                     |position| limited.len() - position,
                 )
         });
@@ -965,6 +1196,7 @@ impl Data {
             new_messages: new.to_vec(),
             max_nick_chars,
             max_prefix_chars,
+            max_excess_timestamp_chars,
             cleared: *cleared,
         })
     }
@@ -1096,16 +1328,22 @@ impl Data {
         self.map.get_mut(kind).and_then(History::make_partial)
     }
 
-    fn flush_all(&mut self, now: Instant) -> Vec<BoxFuture<'static, Message>> {
+    fn flush_all(
+        &mut self,
+        now: Instant,
+        clients: &client::Map,
+    ) -> Vec<BoxFuture<'static, Message>> {
         self.map
             .iter_mut()
             .filter_map(|(kind, state)| {
                 let kind = kind.clone();
 
-                state.flush(Some(now)).map(move |task| {
-                    task.map(move |result| Message::Flushed(kind, result))
-                        .boxed()
-                })
+                state.flush(Some(now), clients.get_seed(&kind)).map(
+                    move |task| {
+                        task.map(move |result| Message::Flushed(kind, result))
+                            .boxed()
+                    },
+                )
             })
             .collect()
     }
