@@ -173,7 +173,7 @@ impl User {
     }
 
     pub fn seed(&self) -> &str {
-        self.as_str()
+        self.nickname.seed()
     }
 
     pub fn display(
@@ -284,6 +284,10 @@ impl User {
         self.away = away;
     }
 
+    pub fn renormalize(&mut self, casemapping: isupport::CaseMap) {
+        self.nickname.renormalize(casemapping);
+    }
+
     pub fn formatted(&self, user_format: UsernameFormat) -> String {
         let user = self.username();
         let host = self.hostname();
@@ -399,12 +403,6 @@ pub enum ParseUserError {
     NicknameEmpty,
 }
 
-#[derive(Debug, Clone)]
-pub struct NickColor {
-    pub seed: Option<String>,
-    pub color: iced_core::Color,
-}
-
 #[derive(Debug, Clone)]
 pub struct Nick {
     raw: String,
@@ -497,6 +495,10 @@ impl<'a> Nick {
     pub fn renormalize(&mut self, casemapping: isupport::CaseMap) {
         self.normalized = casemapping.normalize(self.raw.as_str());
     }
+
+    pub fn seed(&self) -> &str {
+        self.normalized.as_ref()
+    }
 }
 
 impl Serialize for Nick {
