@@ -1,6 +1,5 @@
-use std::io;
 use std::path::PathBuf;
-use std::time;
+use std::{io, time};
 
 use interprocess::local_socket::tokio::LocalSocketListener;
 
@@ -67,7 +66,7 @@ pub fn listen() -> futures::stream::BoxStream<'static, String> {
             State::Uninitialized => match spawn_server().await {
                 Ok(server) => Some((None, State::Waiting(server))),
                 Err(err) => {
-                    println!("error: {err:?}");
+                    log::error!("Unable to spawn server: {err:?}");
                     None
                 }
             },
