@@ -1,12 +1,25 @@
+use std::collections::BTreeMap;
+
 pub use self::command::Command;
 
+pub type Tags = BTreeMap<String, String>;
+
+#[macro_export]
+macro_rules! tags {
+    () => { ::std::collections::BTreeMap::new() };
+    ($x:expr) => { $crate::parse::tagstr($x).unwrap() };
+    ($($key:expr => $value:expr),+ $(,)?) => {
+        ::std::collections::BTreeMap::from([ $((::std::string::String::from($key), ::std::string::String::from($value))),* ])
+    };
+}
+
 pub mod command;
 pub mod format;
 pub mod parse;
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Message {
-    pub tags: Vec<Tag>,
+    pub tags: Tags,
     pub source: Option<Source>,
     pub command: Command,
 }
@@ -14,19 +27,13 @@ pub struct Message {
 impl From<Command> for Message {
     fn from(command: Command) -> Self {
         Self {
-            tags: vec![],
+            tags: tags![],
             source: None,
             command,
         }
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub struct Tag {
-    pub key: String,
-    pub value: Option<String>,
-}
-
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Source {
     Server(String),
@@ -42,7 +49,7 @@ pub struct User {
 
 pub fn command(command: &str, parameters: Vec<String>) -> Message {
     Message {
-        tags: vec![],
+        tags: tags![],
         source: None,
         command: Command::new(command, parameters),
     }
@@ -73,7 +80,7 @@ pub fn parse_channel_from_target(
     chantypes: &[char],
     statusmsg_prefixes: &[char],
 ) -> Option<(Vec<char>, String)> {
-    // We parse the target by finding the first character in chantypes, and returing (even if that
+    // We parse the target by finding the first character in chantypes, and returning (even if that
     // character is in statusmsg_prefixes)
     // If the characters before the first chantypes character are all valid prefixes, then we have
     // a valid channel name with those prefixes.
@@ -100,13 +107,28 @@ macro_rules! command {
     );
 }
 
+// Reference: https://defs.ircdocs.horse/defs/chanmembers
+
+pub const FOUNDER_PREFIX: char = '~';
+pub const PROTECTED_PREFIX_STD: char = '&';
+pub const PROTECTED_PREFIX_ALT: char = '!';
+pub const OPERATOR_PREFIX: char = '@';
+pub const HALF_OPERATOR_PREFIX: char = '%';
+pub const VOICED_PREFIX: char = '+';
+
+pub const CHANNEL_MEMBERSHIP_PREFIXES: &[char] = &[
+    FOUNDER_PREFIX,
+    PROTECTED_PREFIX_STD,
+    PROTECTED_PREFIX_ALT,
+    OPERATOR_PREFIX,
+    HALF_OPERATOR_PREFIX,
+    VOICED_PREFIX,
+];
+
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    // Reference: https://defs.ircdocs.horse/defs/chanmembers
-    const CHANNEL_MEMBERSHIP_PREFIXES: &[char] = &['~', '&', '!', '@', '%', '+'];
-
     #[test]
     fn is_channel_correct() {
         let chantypes = DEFAULT_CHANNEL_PREFIXES;
@@ -143,6 +165,8 @@ mod tests {
     fn invalid_channels() {
         let chantypes = DEFAULT_CHANNEL_PREFIXES;
         let prefixes = CHANNEL_MEMBERSHIP_PREFIXES;
-        assert!(parse_channel_from_target("+%foo", chantypes, prefixes).is_none());
+        assert!(
+            parse_channel_from_target("+%foo", chantypes, prefixes).is_none()
+        );
     }
 }
