@@ -1,5 +1,8 @@
+#[cfg(feature = "tor")]
 use arti_client::{TorClient, TorClientConfig};
-use async_http_proxy::{http_connect_tokio, http_connect_tokio_with_basic_auth};
+use async_http_proxy::{
+    http_connect_tokio, http_connect_tokio_with_basic_auth,
+};
 use fast_socks5::client::{Config as Socks5Config, Socks5Stream};
 use thiserror::Error;
 use tokio::net::TcpStream;
@@ -20,11 +23,16 @@ pub enum Proxy {
         username: Option<String>,
         password: Option<String>,
     },
+    #[cfg(feature = "tor")]
     Tor,
 }
 
 impl Proxy {
-    pub async fn connect(&self, target_server: &str, target_port: u16) -> Result<IrcStream, Error> {
+    pub async fn connect(
+        &self,
+        target_server: &str,
+        target_port: u16,
+    ) -> Result<IrcStream, Error> {
         match self {
             Proxy::Http {
                 host,
@@ -58,7 +66,10 @@ impl Proxy {
                 )
                 .await
             }
-            Proxy::Tor => connect_tor(target_server.to_string(), target_port).await,
+            #[cfg(feature = "tor")]
+            Proxy::Tor => {
+                connect_tor(target_server.to_string(), target_port).await
+            }
         }
     }
 }
@@ -120,7 +131,11 @@ pub async fn connect_socks5(
     Ok(IrcStream::Tcp(stream))
 }
 
-pub async fn connect_tor(target_server: String, target_port: u16) -> Result<IrcStream, Error> {
+#[cfg(feature = "tor")]
+pub async fn connect_tor(
+    target_server: String,
+    target_port: u16,
+) -> Result<IrcStream, Error> {
     let config = TorClientConfig::default();
     let tor_client = TorClient::create_bootstrapped(config).await?;
 
@@ -137,6 +152,7 @@ pub enum Error {
     Io(#[from] std::io::Error),
     #[error("socks5 error: {0}")]
     Socks5(#[from] fast_socks5::SocksError),
+    #[cfg(feature = "tor")]
     #[error("tor error: {0}")]
     Tor(#[from] arti_client::Error),
 }
