@@ -1,10 +1,8 @@
-use iced::{
-    widget::{
-        container,
-        scrollable::{Catalog, Rail, Scroller, Status, Style, StyleFn},
-    },
-    Background, Border, Color, Shadow,
+use iced::widget::container;
+use iced::widget::scrollable::{
+    Catalog, Rail, Scroller, Status, Style, StyleFn,
 };
+use iced::{Background, Border, Color, Shadow};
 
 use super::Theme;
 
@@ -21,11 +19,17 @@ impl Catalog for Theme {
 }
 
 pub fn primary(theme: &Theme, status: Status) -> Style {
+    let scroller_color = theme
+        .styles()
+        .general
+        .scrollbar
+        .unwrap_or(theme.styles().general.horizontal_rule);
+
     let rail = Rail {
         background: None,
         border: Border::default(),
         scroller: Scroller {
-            color: theme.colors().general.horizontal_rule,
+            color: scroller_color,
             border: Border {
                 radius: 8.0.into(),
                 width: 0.0,
@@ -35,7 +39,9 @@ pub fn primary(theme: &Theme, status: Status) -> Style {
     };
 
     match status {
-        Status::Active { .. } | Status::Hovered { .. } | Status::Dragged { .. } => Style {
+        Status::Active { .. }
+        | Status::Hovered { .. }
+        | Status::Dragged { .. } => Style {
             container: container::Style {
                 text_color: None,
                 background: None,
@@ -45,6 +51,7 @@ pub fn primary(theme: &Theme, status: Status) -> Style {
                     color: Color::TRANSPARENT,
                 },
                 shadow: Shadow::default(),
+                snap: true,
             },
             vertical_rail: rail,
             horizontal_rail: rail,
@@ -68,7 +75,9 @@ pub fn hidden(_theme: &Theme, status: Status) -> Style {
     };
 
     match status {
-        Status::Active { .. } | Status::Hovered { .. } | Status::Dragged { .. } => Style {
+        Status::Active { .. }
+        | Status::Hovered { .. }
+        | Status::Dragged { .. } => Style {
             container: container::Style {
                 text_color: None,
                 background: Some(Background::Color(Color::TRANSPARENT)),
@@ -78,6 +87,7 @@ pub fn hidden(_theme: &Theme, status: Status) -> Style {
                     color: Color::TRANSPARENT,
                 },
                 shadow: Shadow::default(),
+                snap: true,
             },
             vertical_rail: rail,
             horizontal_rail: rail,
