@@ -10,7 +10,7 @@ use iced::widget::{column, container, row};
 use iced::{Length, Size, Task, padding};
 
 use super::message_view::{ChannelQueryLayout, TargetInfo};
-use super::{input_view, scroll_view, user_context};
+use super::{context_menu, input_view, scroll_view};
 use crate::Theme;
 use crate::widget::Element;
 
@@ -20,12 +20,12 @@ mod topic;
 pub enum Message {
     ScrollView(scroll_view::Message),
     InputView(input_view::Message),
-    UserContext(user_context::Message),
+    ContextMenu(context_menu::Message),
     Topic(topic::Message),
 }
 
 pub enum Event {
-    UserContext(user_context::Event),
+    ContextMenu(context_menu::Event),
     OpenBuffers(Vec<(Target, BufferAction)>),
     LeaveBuffers(Vec<Target>, Option<String>),
     History(Task<history::manager::Message>),
@@ -54,7 +54,8 @@ pub fn view<'a>(
     let channel = &state.target;
     let buffer = &state.buffer;
     let input = history.input(buffer);
-    let our_nick = clients.nickname(&state.server);
+    let our_nick: Option<data::user::NickRef<'_>> =
+        clients.nickname(&state.server);
 
     let our_user = our_nick
         .map(|our_nick| User::from(Nick::from(our_nick)))
@@ -107,7 +108,7 @@ pub fn view<'a>(
     let nick_list = nick_list::view(
         server, prefix, channel, users, our_user, config, theme,
     )
-    .map(Message::UserContext);
+    .map(Message::ContextMenu);
 
     // If topic toggles from None to Some then it messes with messages' scroll state,
     // so produce a zero-height placeholder when topic is None.
@@ -127,6 +128,7 @@ pub fn view<'a>(
             &state.input_view,
             input,
             is_focused,
+            our_user,
             !is_connected_to_channel,
             config,
             theme,
@@ -134,7 +136,7 @@ pub fn view<'a>(
         .map(Message::InputView)
     });
 
-    let content = column![topic, messages].spacing(4);
+    let content = column![topic, messages];
 
     let nicklist_enabled = settings
         .map_or(config.buffer.channel.nicklist.enabled, |settings| {
@@ -147,16 +149,15 @@ pub fn view<'a>(
             (true, data::channel::Position::Right) => row![content, nick_list],
             (false, _) => { row![content] }.height(Length::Fill),
         }
-        .spacing(4);
+        .spacing(4)
+        .padding(padding::left(8).right(8));
 
     let body = column![container(content).height(Length::Fill), text_input]
-        .spacing(4)
         .height(Length::Fill);
 
     container(body)
         .width(Length::Fill)
         .height(Length::Fill)
-        .padding(padding::all(8).top(4))
         .into()
 }
 
@@ -204,8 +205,8 @@ impl Channel {
                 );
 
                 let event = event.and_then(|event| match event {
-                    scroll_view::Event::UserContext(event) => {
-                        Some(Event::UserContext(event))
+                    scroll_view::Event::ContextMenu(event) => {
+                        Some(Event::ContextMenu(event))
                     }
                     scroll_view::Event::OpenBuffer(target, buffer_action) => {
                         Some(Event::OpenBuffers(vec![(target, buffer_action)]))
@@ -270,15 +271,15 @@ impl Channel {
                     None => (command, None),
                 }
             }
-            Message::UserContext(message) => (
+            Message::ContextMenu(message) => (
                 Task::none(),
-                Some(Event::UserContext(user_context::update(message))),
+                Some(Event::ContextMenu(context_menu::update(message))),
             ),
             Message::Topic(message) => (
                 Task::none(),
                 topic::update(message).map(|event| match event {
-                    topic::Event::UserContext(event) => {
-                        Event::UserContext(event)
+                    topic::Event::ContextMenu(event) => {
+                        Event::ContextMenu(event)
                     }
                     topic::Event::OpenChannel(channel) => {
                         Event::OpenBuffers(vec![(
@@ -311,8 +312,8 @@ fn topic<'a>(
     theme: &'a Theme,
 ) -> Option<Element<'a, Message>> {
     let topic_enabled = settings
-        .map_or(config.buffer.channel.topic.enabled, |settings| {
-            settings.channel.topic.enabled
+        .map_or(config.buffer.channel.topic_banner.enabled, |settings| {
+            settings.channel.topic_banner.enabled
         });
 
     if !topic_enabled {
@@ -335,7 +336,7 @@ fn topic<'a>(
             topic.content.as_ref()?,
             topic.who.as_ref().map(Nick::as_nickref),
             topic.time.as_ref(),
-            config.buffer.channel.topic.max_lines,
+            config.buffer.channel.topic_banner.max_lines,
             users,
             our_user,
             config,
@@ -346,14 +347,14 @@ fn topic<'a>(
 }
 
 mod nick_list {
+    use context_menu::Message;
     use data::user::ChannelUsers;
     use data::{Config, Server, User, config, isupport, target};
     use iced::Length;
     use iced::advanced::text;
     use iced::widget::{Scrollable, column, scrollable};
-    use user_context::Message;
 
-    use crate::buffer::user_context;
+    use crate::buffer::context_menu;
     use crate::widget::{Element, selectable_text};
     use crate::{Theme, font, theme};
 
@@ -406,7 +407,7 @@ mod nick_list {
             })
             .width(Length::Fixed(width));
 
-            user_context::view(
+            context_menu::user(
                 content,
                 server,
                 prefix,
