@@ -5,11 +5,14 @@ use data::dashboard::BufferAction;
 use data::history::{self, ReadMarker};
 use data::input::{self, Cache, RawInput};
 use data::message::server_time;
+use data::rate_limit::TokenPriority;
 use data::target::Target;
 use data::user::Nick;
-use data::{Config, client, command};
-use iced::Task;
-use iced::widget::{column, container, text, text_input};
+use data::{Config, User, client, command};
+use iced::widget::{
+    self, column, container, operation, row, rule, text, text_input,
+};
+use iced::{Alignment, Task, padding};
 use tokio::time;
 
 use self::completion::Completion;
@@ -36,6 +39,7 @@ pub enum Event {
 
 #[derive(Debug, Clone)]
 pub enum Message {
+    SysInfoReceived(iced::system::Information),
     Input(String),
     Send,
     Tab(bool),
@@ -52,6 +56,7 @@ pub fn view<'a>(
     state: &'a State,
     cache: Cache<'a>,
     buffer_focused: bool,
+    our_user: Option<&User>,
     disabled: bool,
     config: &Config,
     theme: &'a Theme,
@@ -65,7 +70,7 @@ pub fn view<'a>(
     let mut text_input = text_input("Send message...", cache.text)
         .on_submit(Message::Send)
         .id(state.input_id.clone())
-        .padding(8)
+        .padding([0, 4])
         .style(style);
 
     if !disabled {
@@ -73,7 +78,7 @@ pub fn view<'a>(
     }
 
     // Add tab support
-    let mut input = key_press(
+    let input = key_press(
         key_press(
             text_input,
             key_press::Key::Named(key_press::Named::Tab),
@@ -85,12 +90,65 @@ pub fn view<'a>(
         Message::Tab(false),
     );
 
+    let our_user_style = {
+        let is_user_away = config
+            .buffer
+            .nickname
+            .away
+            .is_away(our_user.is_none_or(User::is_away));
+
+        let seed = match config.buffer.nickname.color {
+            data::buffer::Color::Solid => None,
+            data::buffer::Color::Unique => {
+                our_user.map(|user| Some(user.seed()))
+            }
+        }
+        .flatten();
+
+        theme::text::nickname(theme, seed, is_user_away, false)
+    };
+
+    let maybe_our_user =
+        config.buffer.text_input.nickname.enabled.then(move || {
+            our_user.map(|user| {
+                container(
+                    text(user.display(
+                        config.buffer.text_input.nickname.show_access_level,
+                        None,
+                    ))
+                    .style(move |_| our_user_style)
+                    .font_maybe(
+                        theme::font_style::nickname(theme, false)
+                            .map(font::get),
+                    ),
+                )
+                .padding(padding::right(4))
+            })
+        });
+
+    let maybe_vertical_rule =
+        maybe_our_user.is_some().then(move || rule::vertical(1.0));
+
+    let mut content = column![
+        container(
+            row![maybe_our_user, maybe_vertical_rule, input]
+                .spacing(4)
+                .height(20.0)
+                .align_y(Alignment::Center)
+        )
+        .padding(8)
+        .style(theme::container::buffer_text_input),
+    ]
+    .spacing(4)
+    .padding(padding::top(4))
+    .into();
+
     // Add up / down support for history cycling
     if buffer_focused {
-        input = key_press(
+        content = key_press(
             key_press(
                 key_press(
-                    input,
+                    content,
                     key_press::Key::Named(key_press::Named::ArrowUp),
                     key_press::Modifiers::default(),
                     Message::Up,
@@ -112,9 +170,10 @@ pub fn view<'a>(
             .as_deref()
             .map(|error_str| error(error_str, theme)),
     ]
+    .padding([0, 8])
     .spacing(4);
 
-    anchored_overlay(input, overlay, anchored_overlay::Anchor::AboveTop, 4.0)
+    anchored_overlay(content, overlay, anchored_overlay::Anchor::AboveTop, 4.0)
 }
 
 fn error<'a, 'b, Message: 'a>(
@@ -133,7 +192,7 @@ fn error<'a, 'b, Message: 'a>(
 
 #[derive(Debug, Clone)]
 pub struct State {
-    input_id: text_input::Id,
+    input_id: widget::Id,
     error: Option<String>,
     completion: Completion,
     selected_history: Option<usize>,
@@ -148,7 +207,7 @@ impl Default for State {
 impl State {
     pub fn new() -> Self {
         Self {
-            input_id: text_input::Id::unique(),
+            input_id: widget::Id::unique(),
             error: None,
             completion: Completion::default(),
             selected_history: None,
@@ -166,6 +225,81 @@ impl State {
         let current_target = buffer.target();
 
         match message {
+            Message::SysInfoReceived(info) => {
+                let sysinfo_config = &config.buffer.commands.sysinfo;
+
+                let sysinfo_parts = [
+                    // OS
+                    sysinfo_config.os.then(|| {
+                        info.system_version.as_deref().map_or_else(
+                            || "OS: Unknown".to_string(),
+                            |version| {
+                                if let Some(kernel) = &info.system_kernel {
+                                    format!("OS: {version} ({kernel})")
+                                } else {
+                                    format!("OS: {version}")
+                                }
+                            },
+                        )
+                    }),
+                    // CPU
+                    sysinfo_config
+                        .cpu
+                        .then(|| format!("CPU: {}", info.cpu_brand.trim())),
+                    // Memory
+                    sysinfo_config.memory.then(|| {
+                        let total_gb = (info.memory_total as f64
+                            / (1024.0 * 1024.0 * 1024.0))
+                            .ceil()
+                            as u64;
+                        format!("MEM: {total_gb} GB")
+                    }),
+                    // GPU
+                    sysinfo_config.gpu.then(|| {
+                        format!(
+                            "GPU: {} ({})",
+                            info.graphics_adapter.trim(),
+                            info.graphics_backend.trim()
+                        )
+                    }),
+                    // Uptime
+                    sysinfo_config
+                        .uptime
+                        .then(|| {
+                            uptime_lib::get().ok().map(|uptime| {
+                                let mut formatter = timeago::Formatter::new();
+                                formatter.num_items(4);
+                                format!("UP: {}", formatter.convert(uptime))
+                            })
+                        })
+                        .flatten(),
+                ]
+                .into_iter()
+                .flatten()
+                .collect::<Vec<_>>();
+
+                // If no sysinfo is enabled, don't send anything
+                if sysinfo_parts.is_empty() {
+                    return (Task::none(), None);
+                }
+
+                let message = sysinfo_parts.join(" ");
+
+                history.record_input_history(buffer, message.clone());
+
+                if let Ok(data::input::Parsed::Input(input)) = input::parse(
+                    buffer.clone(),
+                    config.buffer.text_input.auto_format,
+                    message.as_str(),
+                    clients.nickname(buffer.server()),
+                    &clients.get_isupport(buffer.server()),
+                ) && let Some(encoded) = input.encoded()
+                {
+                    clients.send(buffer, encoded, TokenPriority::User);
+                }
+
+                (Task::none(), None)
+            }
             Message::Input(input) => {
                 // Reset error state
                 self.error = None;
@@ -180,6 +314,8 @@ impl State {
                     .get_channels(buffer.server())
                     .cloned()
                     .collect::<Vec<_>>();
+                let supports_detach =
+                    clients.get_server_supports_detach(buffer.server());
                 let isupport = clients.get_isupport(buffer.server());
 
                 self.completion.process(
@@ -189,6 +325,7 @@ impl State {
                     &history.get_last_seen(buffer),
                     &channels,
                     current_target.as_ref(),
+                    supports_detach,
                     &isupport,
                     config,
                 );
@@ -319,6 +456,18 @@ impl State {
                                         }),
                                     );
                                 }
+                                command::Internal::Detach(channels) => {
+                                    return (
+                                        Task::none(),
+                                        Some(Event::LeaveBuffers {
+                                            targets: channels
+                                                .into_iter()
+                                                .map(Target::Channel)
+                                                .collect(),
+                                            reason: Some("detach".to_string()),
+                                        }),
+                                    );
+                                }
                                 command::Internal::Hop(first, rest) => {
                                     let has_channel_argument = first
                                         .as_ref()
@@ -381,7 +530,11 @@ impl State {
 
                                     // Send part command.
                                     if let Some(part_command) = part_command {
-                                        clients.send(buffer, part_command);
+                                        clients.send(
+                                            buffer,
+                                            part_command,
+                                            TokenPriority::User,
+                                        );
                                     }
 
                                     // Create a delay task that will execute the join after waiting
@@ -450,7 +603,7 @@ impl State {
                                     );
 
                                     let event = history
-                                        .clear_messages(kind)
+                                        .clear_messages(kind, clients)
                                         .map(|history_task| Event::Cleared {
                                             history_task: Task::future(
                                                 history_task,
@@ -459,6 +612,13 @@ impl State {
 
                                     return (Task::none(), event);
                                 }
+                                command::Internal::SysInfo => {
+                                    return (
+                                        iced::system::information()
+                                            .map(Message::SysInfoReceived),
+                                        None,
+                                    );
+                                }
                             }
                         }
                         Ok(input::Parsed::Input(input)) => input,
@@ -473,7 +633,7 @@ impl State {
                     if let Some(encoded) = input.encoded() {
                         let sent_time = server_time(&encoded);
 
-                        clients.send(buffer, encoded);
+                        clients.send(buffer, encoded, TokenPriority::User);
 
                         if config.buffer.mark_as_read.on_message_sent {
                             let chantypes =
@@ -491,6 +651,7 @@ impl State {
                                         buffer.server(),
                                         target,
                                         ReadMarker::from_date_time(sent_time),
+                                        TokenPriority::High,
                                     );
                                 }
                             }
@@ -585,6 +746,8 @@ impl State {
                         .get_channels(buffer.server())
                         .cloned()
                         .collect::<Vec<_>>();
+                    let supports_detach =
+                        clients.get_server_supports_detach(buffer.server());
                     let isupport = clients.get_isupport(buffer.server());
 
                     self.completion.process(
@@ -594,6 +757,7 @@ impl State {
                         &history.get_last_seen(buffer),
                         &channels,
                         current_target.as_ref(),
+                        supports_detach,
                         &isupport,
                         config,
                     );
@@ -629,6 +793,8 @@ impl State {
                             .get_channels(buffer.server())
                             .cloned()
                             .collect::<Vec<_>>();
+                        let supports_detach =
+                            clients.get_server_supports_detach(buffer.server());
                         let isupport = clients.get_isupport(buffer.server());
 
                         self.completion.process(
@@ -638,6 +804,7 @@ impl State {
                             &history.get_last_seen(buffer),
                             &channels,
                             current_target.as_ref(),
+                            supports_detach,
                             &isupport,
                             config,
                         );
@@ -659,7 +826,7 @@ impl State {
 
                 // Send command.
                 if let Some(input) = input {
-                    clients.send(&buffer, input);
+                    clients.send(&buffer, input, TokenPriority::User);
                 }
 
                 (Task::none(), None)
@@ -686,17 +853,17 @@ impl State {
             });
         }
 
-        (text_input::move_cursor_to_end(self.input_id.clone()), None)
+        (operation::move_cursor_to_end(self.input_id.clone()), None)
     }
 
     pub fn focus(&self) -> Task<Message> {
         let input_id = self.input_id.clone();
 
-        text_input::is_focused(input_id.clone()).then(move |is_focused| {
+        operation::is_focused(input_id.clone()).then(move |is_focused| {
             if is_focused {
                 Task::none()
             } else {
-                text_input::focus(input_id.clone())
+                operation::focus(input_id.clone())
             }
         })
     }
@@ -738,7 +905,7 @@ impl State {
 
         history.record_draft(RawInput { buffer, text });
 
-        text_input::move_cursor_to_end(self.input_id.clone())
+        operation::move_cursor_to_end(self.input_id.clone())
     }
 
     pub fn close_picker(&mut self) -> bool {
