@@ -44,6 +44,7 @@ impl Completion {
         last_seen: &HashMap<Nick, DateTime<Utc>>,
         channels: &[target::Channel],
         current_target: Option<&Target>,
+        supports_detach: bool,
         isupport: &HashMap<isupport::Kind, isupport::Parameter>,
         config: &Config,
     ) {
@@ -63,6 +64,7 @@ impl Completion {
                 input,
                 our_nickname,
                 current_target,
+                supports_detach,
                 isupport,
             );
 
@@ -290,6 +292,7 @@ impl Commands {
         input: &str,
         our_nickname: Option<NickRef>,
         current_target: Option<&Target>,
+        supports_detach: bool,
         isupport: &HashMap<isupport::Kind, isupport::Parameter>,
     ) {
         let Some((head, rest)) = input.split_once('/') else {
@@ -564,6 +567,14 @@ impl Commands {
                     subcommands: None,
                 }
             },
+            // SYSINFO
+            {
+                Command {
+                    title: "SYSINFO",
+                    args: vec![],
+                    subcommands: None,
+                }
+            },
             // CLEAR
             {
                 Command {
@@ -623,6 +634,7 @@ impl Commands {
                            \n      PING: Request a reply containing the same <info> that was sent\
                            \n    SOURCE: Request a URL where the source code for <nick>'s IRC client can be found\
                            \n      TIME: Request the <nick>'s local time in a human-readable format\
+                           \n  USERINFO: Request miscellaneous information about the user\
                            \n   VERSION: Request the name and version of <nick>'s IRC client".to_string(),
                         ),
                     },
@@ -630,6 +642,7 @@ impl Commands {
                 subcommands: Some(vec![
                         ctcp_action_command(),
                         ctcp_clientinfo_command(),
+                        ctcp_userinfo_command(),
                         ctcp_ping_command(),
                         ctcp_source_command(),
                         ctcp_time_command(),
@@ -639,6 +652,19 @@ impl Commands {
             },
         ];
 
+        if supports_detach {
+            let default = current_target
+                .and_then(|target| target.as_channel())
+                .map(target::Channel::to_string);
+
+            let channel_len = match isupport.get(&isupport::Kind::CHANNELLEN) {
+                Some(isupport::Parameter::CHANNELLEN(len)) => Some(*len),
+                _ => None,
+            };
+
+            command_list.push(detach_command(default, channel_len));
+        }
+
         let isupport_commands = isupport
             .iter()
             .filter_map(|(_, isupport_parameter)| match isupport_parameter {
@@ -986,6 +1012,10 @@ impl Command {
             "hop" => "Parts the current channel and joins a new one",
             "clear" => "Clears the buffer",
             "cleartopic" => "Clear the topic of a channel",
+            "sysinfo" => "Send system information",
+            "detach" => {
+                "Hide the channel, leaving the bouncer's connection to the channel active"
+            }
             _ => return None,
         })
     }
@@ -1006,6 +1036,7 @@ impl Command {
             "format" => vec!["f"],
             "hop" => vec!["rejoin"],
             "clear" => vec![],
+            "sysinfo" => vec![],
             _ => vec![],
         }
     }
@@ -1440,6 +1471,14 @@ fn ctcp_clientinfo_command() -> Command {
     }
 }
 
+fn ctcp_userinfo_command() -> Command {
+    Command {
+        title: "CTCP USERINFO",
+        args: vec![],
+        subcommands: None,
+    }
+}
+
 fn ctcp_ping_command() -> Command {
     Command {
         title: "CTCP PING",
@@ -1750,6 +1789,39 @@ static CPRIVMSG_COMMAND: LazyLock<Command> = LazyLock::new(|| Command {
     subcommands: None,
 });
 
+fn detach_command(
+    default: Option<String>,
+    channel_len: Option<u16>,
+) -> Command {
+    let mut channels_tooltip = String::from("comma-separated");
+
+    if let Some(channel_len) = channel_len {
+        channels_tooltip.push_str(
+            format!("\nmaximum length of each: {channel_len}").as_str(),
+        );
+    }
+
+    if let Some(default) = &default {
+        channels_tooltip.push_str(
+            format!("\nmay be skipped (default: {default})").as_str(),
+        );
+    }
+
+    Command {
+        title: "DETACH",
+        args: vec![Argument {
+            text: "channels",
+            kind: if default.is_some() {
+                ArgumentKind::Optional { skipped: false }
+            } else {
+                ArgumentKind::Required
+            },
+            tooltip: Some(channels_tooltip),
+        }],
+        subcommands: None,
+    }
+}
+
 fn join_command(
     channel_len: Option<u16>,
     channel_limits: Option<&Vec<isupport::ChannelLimit>>,
