@@ -3,12 +3,13 @@ use std::path::PathBuf;
 use data::dashboard::BufferAction;
 use data::preview::{self, Previews};
 use data::target::{self, Target};
-use data::{Config, Server, buffer, history, message};
-use iced::widget::{column, container, vertical_space};
+use data::user::Nick;
+use data::{Config, Server, User, buffer, history, message};
+use iced::widget::{column, container, space};
 use iced::{Length, Size, Task};
 
 use super::message_view::{ChannelQueryLayout, TargetInfo};
-use super::{input_view, scroll_view, user_context};
+use super::{context_menu, input_view, scroll_view};
 use crate::Theme;
 use crate::widget::Element;
 
@@ -19,7 +20,7 @@ pub enum Message {
 }
 
 pub enum Event {
-    UserContext(user_context::Event),
+    ContextMenu(context_menu::Event),
     OpenBuffers(Vec<(Target, BufferAction)>),
     LeaveBuffers(Vec<Target>, Option<String>),
     History(Task<history::manager::Message>),
@@ -48,6 +49,8 @@ pub fn view<'a>(
     let status = clients.status(server);
     let buffer = &state.buffer;
     let input = history.input(buffer);
+    let our_nick = clients.nickname(&state.server);
+    let our_user = our_nick.map(|our_nick| User::from(Nick::from(our_nick)));
 
     let chathistory_state =
         clients.get_chathistory_state(server, &query.to_target());
@@ -91,11 +94,12 @@ pub fn view<'a>(
 
     let text_input = show_text_input.then(|| {
         column![
-            vertical_space().height(4),
+            space::vertical().height(4),
             input_view::view(
                 &state.input_view,
                 input,
                 is_focused,
+                our_user.as_ref(),
                 !status.connected(),
                 config,
                 theme,
@@ -105,12 +109,11 @@ pub fn view<'a>(
         .width(Length::Fill)
     });
 
-    let scrollable = column![messages, text_input,].height(Length::Fill);
+    let scrollable = column![messages, text_input].height(Length::Fill);
 
     container(scrollable)
         .width(Length::Fill)
         .height(Length::Fill)
-        .padding(8)
         .into()
 }
 
@@ -158,8 +161,8 @@ impl Query {
                 );
 
                 let event = event.and_then(|event| match event {
-                    scroll_view::Event::UserContext(event) => {
-                        Some(Event::UserContext(event))
+                    scroll_view::Event::ContextMenu(event) => {
+                        Some(Event::ContextMenu(event))
                     }
                     scroll_view::Event::OpenBuffer(target, buffer_action) => {
                         Some(Event::OpenBuffers(vec![(target, buffer_action)]))
