@@ -1,5 +1,6 @@
 use std::collections::HashMap;
 use std::path::PathBuf;
+use std::time::Duration;
 
 use chrono::{DateTime, Local, NaiveDate, NaiveTime, Utc};
 use data::buffer::DateSeparators;
@@ -10,17 +11,16 @@ use data::preview::{self, Previews};
 use data::server::Server;
 use data::target::{self, Target};
 use data::{Config, Preview, client, history};
-use iced::widget::text::LineHeight;
 use iced::widget::{
-    Scrollable, button, center, column, container, horizontal_rule,
-    horizontal_space, image, mouse_area, right, row, scrollable, stack, text,
+    self, Scrollable, button, center, column, container, image, mouse_area,
+    right, row, rule, scrollable, space, stack, text,
 };
 use iced::{ContentFit, Length, Padding, Size, Task, alignment, padding};
+use tokio::time;
 
 use self::correct_viewport::correct_viewport;
 use self::keyed::keyed;
-use super::user_context;
-use crate::appearance::theme::TEXT_SIZE;
+use super::context_menu;
 use crate::widget::{
     Element, MESSAGE_MARKER_TEXT, notify_visibility, on_resize,
     selectable_text, tooltip,
@@ -28,6 +28,7 @@ use crate::widget::{
 use crate::{Theme, font, icon, theme};
 
 const HIDE_BUTTON_WIDTH: f32 = 22.0;
+const SCROLL_TO_TIMEOUT: Duration = Duration::from_millis(200);
 
 #[derive(Debug, Clone)]
 pub enum Message {
@@ -39,7 +40,7 @@ pub enum Message {
         status: Status,
         viewport: scrollable::Viewport,
     },
-    UserContext(user_context::Message),
+    ContextMenu(context_menu::Message),
     Link(message::Link),
     ImagePreview(PathBuf, url::Url),
     ScrollTo(keyed::Hit),
@@ -51,11 +52,12 @@ pub enum Message {
     HidePreview(message::Hash, url::Url),
     MarkAsRead,
     ContentResized(Size),
+    PendingScrollTo,
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
-    UserContext(user_context::Event),
+    ContextMenu(context_menu::Event),
     OpenBuffer(Target, BufferAction),
     GoToMessage(Server, target::Channel, message::Hash),
     RequestOlderChatHistory,
@@ -105,9 +107,10 @@ impl From<Kind<'_>> for history::Kind {
 pub trait LayoutMessage<'a> {
     fn format(
         &self,
-        msg: &'a data::Message,
+        message: &'a data::Message,
         max_nick_width: Option<f32>,
         max_prefix_width: Option<f32>,
+        max_excess_timestamp_width: Option<f32>,
     ) -> Option<Element<'a, Message>>;
 }
 
@@ -117,15 +120,22 @@ where
         &'a data::Message,
         Option<f32>,
         Option<f32>,
+        Option<f32>,
     ) -> Option<Element<'a, Message>>,
 {
     fn format(
         &self,
-        msg: &'a data::Message,
+        message: &'a data::Message,
         max_nick_width: Option<f32>,
         max_prefix_width: Option<f32>,
+        max_excess_timestamp_width: Option<f32>,
     ) -> Option<Element<'a, Message>> {
-        self(msg, max_nick_width, max_prefix_width)
+        self(
+            message,
+            max_nick_width,
+            max_prefix_width,
+            max_excess_timestamp_width,
+        )
     }
 }
 
@@ -149,6 +159,7 @@ pub fn view<'a>(
         new_messages,
         max_nick_chars,
         max_prefix_chars,
+        max_excess_timestamp_chars,
         cleared,
         ..
     }) = history.get_messages(&kind.into(), Some(state.limit), &config.buffer)
@@ -177,7 +188,7 @@ pub fn view<'a>(
             .on_press_maybe(message);
 
         Some(
-            row![horizontal_space(), top_row_button, horizontal_space()]
+            row![space::horizontal(), top_row_button, space::horizontal()]
                 .padding(padding::top(2).bottom(6))
                 .width(Length::Fill)
                 .align_y(iced::Alignment::Center),
@@ -196,7 +207,13 @@ pub fn view<'a>(
 
     let max_nick_width = max_nick_chars.map(|len| {
         font::width_from_chars(
-            usize::max(len, MESSAGE_MARKER_TEXT.chars().count()),
+            usize::max(
+                len,
+                usize::max(
+                    max_excess_timestamp_chars.unwrap_or_default(),
+                    MESSAGE_MARKER_TEXT.chars().count(),
+                ),
+            ),
             &config.font,
         )
     });
@@ -204,13 +221,21 @@ pub fn view<'a>(
     let max_prefix_width =
         max_prefix_chars.map(|len| font::width_from_chars(len, &config.font));
 
+    let max_excess_timestamp_width = max_excess_timestamp_chars
+        .map(|len| font::width_from_chars(len, &config.font));
+
     let message_rows = |last_date: Option<NaiveDate>,
                         messages: &[&'a data::Message]| {
         messages
             .iter()
             .filter_map(|message| {
                 formatter
-                    .format(message, max_nick_width, max_prefix_width)
+                    .format(
+                        message,
+                        max_nick_width,
+                        max_prefix_width,
+                        max_excess_timestamp_width,
+                    )
                     .map(|element| {
                         (message, keyed(keyed::Key::message(message), element))
                     })
@@ -241,36 +266,17 @@ pub fn view<'a>(
                             .visible_url_messages
                             .contains_key(&message.hash);
 
-                        let element = if is_message_visible {
-                            notify_visibility(
-                                element,
-                                2000.0,
-                                notify_visibility::When::NotVisible,
-                                Message::ExitingViewport(message.hash),
-                            )
-                        } else {
-                            notify_visibility(
-                                element,
-                                1000.0,
-                                notify_visibility::When::Visible,
-                                Message::EnteringViewport(
-                                    message.hash,
-                                    urls.clone(),
-                                ),
-                            )
-                        };
-
                         let mut column = column![element];
 
-                        for (idx, url) in urls.into_iter().enumerate() {
-                            if message.hidden_urls.contains(&url) {
+                        for (idx, url) in urls.iter().enumerate() {
+                            if message.hidden_urls.contains(url) {
                                 continue;
                             }
 
                             if let (
                                 true,
                                 Some(preview::State::Loaded(preview)),
-                            ) = (is_message_visible, previews.get(&url))
+                            ) = (is_message_visible, previews.get(url))
                             {
                                 let is_hovered =
                                     state.hovered_preview.is_some_and(
@@ -280,7 +286,7 @@ pub fn view<'a>(
                                 column = column.push(preview_row(
                                     message,
                                     preview,
-                                    &url,
+                                    url,
                                     idx,
                                     max_nick_width,
                                     max_prefix_width,
@@ -291,7 +297,21 @@ pub fn view<'a>(
                             }
                         }
 
-                        column.into()
+                        if is_message_visible {
+                            notify_visibility(
+                                column,
+                                2000.0,
+                                notify_visibility::When::NotVisible,
+                                Message::ExitingViewport(message.hash),
+                            )
+                        } else {
+                            notify_visibility(
+                                column,
+                                1000.0,
+                                notify_visibility::When::Visible,
+                                Message::EnteringViewport(message.hash, urls),
+                            )
+                        }
                     } else {
                         element
                     }
@@ -303,7 +323,7 @@ pub fn view<'a>(
                     Some(
                         column![
                             row![
-                                container(horizontal_rule(1))
+                                container(rule::horizontal(1))
                                     .width(Length::Fill)
                                     .padding(padding::right(6)),
                                 text(
@@ -336,7 +356,7 @@ pub fn view<'a>(
                                     theme::font_style::secondary(theme)
                                         .map(font::get)
                                 ),
-                                container(horizontal_rule(1))
+                                container(rule::horizontal(1))
                                     .width(Length::Fill)
                                     .padding(padding::left(6))
                             ]
@@ -376,14 +396,14 @@ pub fn view<'a>(
 
     let divider = if show_backlog_divier {
         row![
-            container(horizontal_rule(1))
+            container(rule::horizontal(1))
                 .width(Length::Fill)
                 .padding(padding::right(6)),
             text("backlog")
                 .size(divider_font_size)
                 .style(theme::text::secondary)
                 .font_maybe(theme::font_style::secondary(theme).map(font::get)),
-            container(horizontal_rule(1))
+            container(rule::horizontal(1))
                 .width(Length::Fill)
                 .padding(padding::left(6))
         ]
@@ -399,6 +419,7 @@ pub fn view<'a>(
             column(old).spacing(config.buffer.line_spacing),
             keyed(keyed::Key::Divider, divider),
             column(new).spacing(config.buffer.line_spacing),
+            space::vertical().height(config.buffer.line_spacing),
         ]
         .spacing(config.buffer.line_spacing),
         Message::ContentResized,
@@ -428,41 +449,31 @@ pub fn view<'a>(
 
 #[derive(Debug, Clone)]
 pub struct State {
-    pub scrollable: scrollable::Id,
+    pub scrollable: widget::Id,
     pane_size: Size,
     content_size: Size,
     limit: Limit,
     status: Status,
-    pending_scroll_to: Option<message::Hash>,
+    pending_scroll_to: Option<keyed::Key>,
     visible_url_messages: HashMap<message::Hash, Vec<url::Url>>,
     hovered_preview: Option<(message::Hash, usize)>,
 }
 
-impl Default for State {
-    fn default() -> Self {
+impl State {
+    pub fn new(pane_size: Size, config: &Config) -> Self {
+        let step_messages = step_messages(2.0 * pane_size.height, config);
+
         Self {
-            scrollable: scrollable::Id::unique(),
-            pane_size: Size::default(), // Will get set initially via `update_size`
+            scrollable: widget::Id::unique(),
+            pane_size,
             content_size: Size::default(), // Will get set initially via `on_resize`
-            limit: Limit::Bottom(0),
+            limit: Limit::Bottom(step_messages),
             status: Status::default(),
             pending_scroll_to: None,
             visible_url_messages: HashMap::new(),
             hovered_preview: None,
         }
     }
-}
-
-impl State {
-    pub fn new(pane_size: Size, config: &Config) -> Self {
-        let step_messages = step_messages(pane_size.height, config);
-
-        Self {
-            pane_size,
-            limit: Limit::Bottom(step_messages),
-            ..Self::default()
-        }
-    }
 
     pub fn update(
         &mut self,
@@ -514,7 +525,10 @@ impl State {
                         if matches!(self.limit, Limit::Bottom(_)) {
                             if old_status.is_page_from_top(
                                 absolute_offset,
-                                height,
+                                // Scale up page height to ensure that there
+                                // isn't a simultaneous anchor flip and message
+                                // load when scrolling up from bottom
+                                2.0 * height,
                                 self.content_size.height,
                             ) && has_more_older_messages
                             {
@@ -523,8 +537,10 @@ impl State {
                                 );
                             }
                         } else {
-                            self.limit =
-                                Limit::Bottom(step_messages(height, config));
+                            self.limit = Limit::Bottom(step_messages(
+                                2.0 * height,
+                                config,
+                            ));
                         }
                     }
                     // Scrolling up from bottom & have more to load
@@ -584,8 +600,10 @@ impl State {
                                     );
                                 }
                             } else {
-                                self.limit =
-                                    Limit::Top(step_messages(height, config));
+                                self.limit = Limit::Top(step_messages(
+                                    2.0 * height,
+                                    config,
+                                ));
                             }
                         }
                     }
@@ -619,10 +637,10 @@ impl State {
 
                 return (Task::batch(tasks), event);
             }
-            Message::UserContext(message) => {
+            Message::ContextMenu(message) => {
                 return (
                     Task::none(),
-                    Some(Event::UserContext(user_context::update(message))),
+                    Some(Event::ContextMenu(context_menu::update(message))),
                 );
             }
             Message::Link(message::Link::Channel(channel)) => {
@@ -648,7 +666,7 @@ impl State {
                         )
                     }
                     data::config::buffer::NicknameClickAction::InsertNickname => {
-                        Event::UserContext(user_context::Event::InsertNickname(
+                        Event::ContextMenu(context_menu::Event::InsertNickname(
                             user.nickname().to_owned(),
                         ))
                     }
@@ -689,7 +707,7 @@ impl State {
 
                     if !matches!(self.limit, Limit::Bottom(_)) {
                         self.limit = Limit::Bottom(step_messages(
-                            self.pane_size.height,
+                            2.0 * self.pane_size.height,
                             config,
                         ));
                     }
@@ -758,10 +776,28 @@ impl State {
             }
             Message::ContentResized(size) => {
                 self.content_size = size;
+
+                if let Some(key) = &self.pending_scroll_to {
+                    let scroll_to = keyed::find(self.scrollable.clone(), *key)
+                        .map(Message::ScrollTo);
+
+                    self.pending_scroll_to = None;
+                    return (scroll_to, None);
+                }
             }
             Message::ImagePreview(path, url) => {
                 return (Task::none(), Some(Event::ImagePreview(path, url)));
             }
+            Message::PendingScrollTo => {
+                if let Some(key) = &self.pending_scroll_to {
+                    println!("pending_scroll_to → Task");
+                    let scroll_to = keyed::find(self.scrollable.clone(), *key)
+                        .map(Message::ScrollTo);
+
+                    self.pending_scroll_to = None;
+                    return (scroll_to, None);
+                }
+            }
         }
 
         (Task::none(), None)
@@ -807,7 +843,8 @@ impl State {
 
     pub fn scroll_to_start(&mut self, config: &Config) -> Task<Message> {
         self.status = Status::Unlocked;
-        self.limit = Limit::Top(step_messages(self.pane_size.height, config));
+        self.limit =
+            Limit::Top(step_messages(2.0 * self.pane_size.height, config));
         correct_viewport::scroll_to(
             self.scrollable.clone(),
             scrollable::AbsoluteOffset { x: 0.0, y: 0.0 },
@@ -817,7 +854,7 @@ impl State {
     pub fn scroll_to_end(&mut self, config: &Config) -> Task<Message> {
         self.status = Status::Bottom;
         self.limit =
-            Limit::Bottom(step_messages(self.pane_size.height, config));
+            Limit::Bottom(step_messages(2.0 * self.pane_size.height, config));
         correct_viewport::scroll_to(
             self.scrollable.clone(),
             scrollable::AbsoluteOffset { x: 0.0, y: 0.0 },
@@ -842,10 +879,9 @@ impl State {
             ..
         }) = history.get_messages(&kind.into(), None, &config.buffer)
         else {
-            // We're still loading history, which will trigger
-            // scroll_to_backlog after loading. If this is set,
-            // we will scroll_to_message
-            self.pending_scroll_to = Some(message);
+            // We're still loading history, which will trigger scroll_to_backlog
+            // after loading. If this is set, we will scroll_to_message
+            self.pending_scroll_to = Some(keyed::Key::Message(message));
 
             return Task::none();
         };
@@ -862,11 +898,14 @@ impl State {
         let offset = total - pos + 1;
 
         self.limit = Limit::Bottom(
-            offset.max(step_messages(self.pane_size.height, config)),
+            offset.max(step_messages(2.0 * self.pane_size.height, config)),
         );
 
-        keyed::find(self.scrollable.clone(), keyed::Key::Message(message))
-            .map(Message::ScrollTo)
+        self.pending_scroll_to = Some(keyed::Key::Message(message));
+
+        Task::perform(time::sleep(SCROLL_TO_TIMEOUT), move |()| {
+            Message::PendingScrollTo
+        })
     }
 
     pub fn scroll_to_backlog(
@@ -875,12 +914,10 @@ impl State {
         history: &history::Manager,
         config: &Config,
     ) -> Task<Message> {
-        if let Some(message) = self.pending_scroll_to.take() {
-            return self.scroll_to_message(message, kind, history, config);
-        }
-
-        if history.read_marker(&kind.into()).is_none() {
-            return Task::none();
+        if self.pending_scroll_to.is_some() {
+            return Task::perform(time::sleep(SCROLL_TO_TIMEOUT), move |()| {
+                Message::PendingScrollTo
+            });
         }
 
         let Some(history::View {
@@ -896,11 +933,14 @@ impl State {
         let offset = total - old_messages.len() + 1;
 
         self.limit = Limit::Bottom(
-            offset.max(step_messages(self.pane_size.height, config)),
+            offset.max(step_messages(2.0 * self.pane_size.height, config)),
         );
 
-        keyed::find(self.scrollable.clone(), keyed::Key::Divider)
-            .map(Message::ScrollTo)
+        self.pending_scroll_to = Some(keyed::Key::Divider);
+
+        Task::perform(time::sleep(SCROLL_TO_TIMEOUT), move |()| {
+            Message::PendingScrollTo
+        })
     }
 
     pub fn visible_urls(&self) -> impl Iterator<Item = &url::Url> {
@@ -990,16 +1030,7 @@ impl Default for Status {
 }
 
 fn step_messages(height: f32, config: &Config) -> usize {
-    let line_height = LineHeight::default()
-        .to_absolute(
-            if let Some(size) = config.font.size {
-                f32::from(size)
-            } else {
-                TEXT_SIZE
-            }
-            .into(),
-        )
-        .0;
+    let line_height = theme::line_height(&config.font);
 
     (height / line_height) as usize
 }
@@ -1085,7 +1116,7 @@ mod keyed {
         }
     }
 
-    pub fn find(scrollable: scrollable::Id, key: Key) -> Task<Hit> {
+    pub fn find(scrollable: widget::Id, key: Key) -> Task<Hit> {
         widget::operate(Find {
             active: false,
             scrollable_id: scrollable,
@@ -1100,7 +1131,7 @@ mod keyed {
     pub struct Find {
         pub active: bool,
         pub key: Key,
-        pub scrollable_id: scrollable::Id,
+        pub scrollable_id: widget::Id,
         pub scrollable: Option<Scrollable>,
         pub hit_bounds: Option<Rectangle>,
         pub prev_bounds: Option<Rectangle>,
@@ -1115,7 +1146,7 @@ mod keyed {
             translation: Vector,
             _state: &mut dyn widget::operation::Scrollable,
         ) {
-            if id == Some(&self.scrollable_id.clone().into()) {
+            if id == Some(&self.scrollable_id.clone()) {
                 self.scrollable = Some(Scrollable {
                     viewport: bounds,
                     content: content_bounds,
@@ -1130,13 +1161,13 @@ mod keyed {
             }
         }
 
-        fn container(
+        fn container(&mut self, _id: Option<&widget::Id>, _bounds: Rectangle) {}
+
+        fn traverse(
             &mut self,
-            _id: Option<&widget::Id>,
-            _bounds: Rectangle,
-            operate_on_children: &mut dyn FnMut(&mut dyn Operation<Hit>),
+            operate: &mut dyn FnMut(&mut dyn Operation<Hit>),
         ) {
-            operate_on_children(self);
+            operate(self);
         }
 
         fn custom(
@@ -1174,7 +1205,7 @@ mod keyed {
     #[derive(Debug, Clone)]
     pub struct TopOfViewport {
         pub active: bool,
-        pub scrollable_id: scrollable::Id,
+        pub scrollable_id: widget::Id,
         pub scrollable: Option<Scrollable>,
         pub hit_bounds: Option<(Key, Rectangle)>,
     }
@@ -1188,7 +1219,7 @@ mod keyed {
             translation: Vector,
             _state: &mut dyn widget::operation::Scrollable,
         ) {
-            if id == Some(&self.scrollable_id.clone().into()) {
+            if id == Some(&self.scrollable_id.clone()) {
                 self.scrollable = Some(Scrollable {
                     viewport: bounds,
                     content: content_bounds,
@@ -1203,13 +1234,13 @@ mod keyed {
             }
         }
 
-        fn container(
+        fn container(&mut self, _id: Option<&widget::Id>, _bounds: Rectangle) {}
+
+        fn traverse(
             &mut self,
-            _id: Option<&widget::Id>,
-            _bounds: Rectangle,
-            operate_on_children: &mut dyn FnMut(&mut dyn Operation<Hit>),
+            operate: &mut dyn FnMut(&mut dyn Operation<Hit>),
         ) {
-            operate_on_children(self);
+            operate(self);
         }
 
         fn custom(
@@ -1412,7 +1443,7 @@ fn preview_row<'a>(
             theme,
         ))
     } else {
-        container(horizontal_space().width(Length::Fixed(HIDE_BUTTON_WIDTH)))
+        container(space::horizontal().width(Length::Fixed(HIDE_BUTTON_WIDTH)))
     };
 
     // Iced hack: using a stack with right-aligned hide_button ensures the button always stays visible
@@ -1449,7 +1480,7 @@ mod correct_viewport {
 
     pub fn correct_viewport<'a>(
         inner: impl Into<Element<'a, Message>>,
-        scrollable: iced::widget::scrollable::Id,
+        scrollable: iced::widget::Id,
         enabled: bool,
     ) -> Element<'a, Message> {
         decorate(inner)
@@ -1516,7 +1547,7 @@ mod correct_viewport {
                                 );
 
                                 let mut operation = scrollable::scroll_to(
-                                    scrollable.clone().into(),
+                                    scrollable.clone(),
                                     scrollable::AbsoluteOffset {
                                         x: 0.0,
                                         y: new_offset,
@@ -1617,7 +1648,7 @@ mod correct_viewport {
                     let mut is_scroll_to = false;
 
                     operation.custom(
-                        Some(&scrollable.clone().into()),
+                        Some(&scrollable.clone()),
                         layout.bounds(),
                         &mut is_scroll_to,
                     );
@@ -1665,13 +1696,13 @@ mod correct_viewport {
         }
 
         impl<T> Operation<T> for ScrollTo {
-            fn container(
+            fn container(&mut self, _id: Option<&Id>, _bounds: Rectangle) {}
+
+            fn traverse(
                 &mut self,
-                _id: Option<&Id>,
-                _bounds: Rectangle,
-                operate_on_children: &mut dyn FnMut(&mut dyn Operation<T>),
+                operate: &mut dyn FnMut(&mut dyn Operation<T>),
             ) {
-                operate_on_children(self);
+                operate(self);
             }
 
             fn scrollable(
@@ -1682,7 +1713,7 @@ mod correct_viewport {
                 _translation: Vector,
                 state: &mut dyn Scrollable,
             ) {
-                if Some(&self.target) == id {
+                if id.is_some_and(|id| *id == self.target) {
                     state.scroll_to(self.offset);
                 }
             }
@@ -1719,13 +1750,13 @@ mod correct_viewport {
         }
 
         impl<T> Operation<T> for ScrollBy {
-            fn container(
+            fn container(&mut self, _id: Option<&Id>, _bounds: Rectangle) {}
+
+            fn traverse(
                 &mut self,
-                _id: Option<&Id>,
-                _bounds: Rectangle,
-                operate_on_children: &mut dyn FnMut(&mut dyn Operation<T>),
+                operate: &mut dyn FnMut(&mut dyn Operation<T>),
             ) {
-                operate_on_children(self);
+                operate(self);
             }
 
             fn scrollable(
