@@ -2,11 +2,12 @@ use std::path::PathBuf;
 
 use data::dashboard::BufferAction;
 use data::target::Target;
-use data::{Config, buffer, history, message};
-use iced::widget::{column, container, row, vertical_space};
-use iced::{Length, Size, Task, padding};
+use data::user::Nick;
+use data::{Config, User, buffer, history, message};
+use iced::widget::{column, container, row, space};
+use iced::{Color, Length, Size, Task};
 
-use super::{input_view, scroll_view, user_context};
+use super::{context_menu, input_view, scroll_view};
 use crate::widget::{Element, message_content, selectable_text};
 use crate::{Theme, font, theme};
 
@@ -17,7 +18,7 @@ pub enum Message {
 }
 
 pub enum Event {
-    UserContext(user_context::Event),
+    ContextMenu(context_menu::Event),
     OpenBuffers(Vec<(Target, BufferAction)>),
     LeaveBuffers(Vec<Target>, Option<String>),
     History(Task<history::manager::Message>),
@@ -39,6 +40,9 @@ pub fn view<'a>(
     let casemapping = clients.get_casemapping(&state.server);
     let buffer = &state.buffer;
     let input = history.input(buffer);
+    let our_nick: Option<data::user::NickRef<'_>> =
+        clients.nickname(&state.server);
+    let our_user = our_nick.map(|our_nick| User::from(Nick::from(our_nick)));
 
     let messages = container(
         scroll_view::view(
@@ -49,17 +53,23 @@ pub fn view<'a>(
             None,
             config,
             theme,
-            move |message: &'a data::Message, _, _| {
+            move |message: &'a data::Message, _, _, _| {
                 let timestamp = config
                     .buffer
                     .format_timestamp(&message.server_time)
                     .map(|timestamp| {
-                        selectable_text(timestamp)
-                            .font_maybe(
-                                theme::font_style::timestamp(theme)
-                                    .map(font::get),
-                            )
-                            .style(theme::selectable_text::timestamp)
+                        context_menu::timestamp(
+                            selectable_text(timestamp)
+                                .font_maybe(
+                                    theme::font_style::timestamp(theme)
+                                        .map(font::get),
+                                )
+                                .style(theme::selectable_text::timestamp),
+                            &message.server_time,
+                            config,
+                            theme,
+                        )
+                        .map(scroll_view::Message::ContextMenu)
                     });
 
                 match message.target.source() {
@@ -82,10 +92,18 @@ pub fn view<'a>(
                                     server.as_ref(),
                                 )
                             },
+                            Option::<fn(Color) -> Color>::None,
                             config,
                         );
 
-                        Some(container(row![timestamp, message]).into())
+                        Some(
+                            container(row![
+                                timestamp,
+                                selectable_text(" "),
+                                message
+                            ])
+                            .into(),
+                        )
                     }
                     message::Source::Internal(
                         message::source::Internal::Status(status),
@@ -102,10 +120,18 @@ pub fn view<'a>(
                             move |theme| {
                                 theme::font_style::status(theme, *status)
                             },
+                            Option::<fn(Color) -> Color>::None,
                             config,
                         );
 
-                        Some(container(row![timestamp, message]).into())
+                        Some(
+                            container(row![
+                                timestamp,
+                                selectable_text(" "),
+                                message
+                            ])
+                            .into(),
+                        )
                     }
                     _ => None,
                 }
@@ -122,11 +148,12 @@ pub fn view<'a>(
 
     let text_input = show_text_input.then(|| {
         column![
-            vertical_space().height(4),
+            space::vertical().height(4),
             input_view::view(
                 &state.input_view,
                 input,
                 is_focused,
+                our_user.as_ref(),
                 !status.connected(),
                 config,
                 theme,
@@ -141,7 +168,6 @@ pub fn view<'a>(
     container(scrollable)
         .width(Length::Fill)
         .height(Length::Fill)
-        .padding(padding::all(8).top(4))
         .into()
 }
 
@@ -186,8 +212,8 @@ impl Server {
                 );
 
                 let event = event.and_then(|event| match event {
-                    scroll_view::Event::UserContext(event) => {
-                        Some(Event::UserContext(event))
+                    scroll_view::Event::ContextMenu(event) => {
+                        Some(Event::ContextMenu(event))
                     }
                     scroll_view::Event::OpenBuffer(target, buffer_action) => {
                         Some(Event::OpenBuffers(vec![(target, buffer_action)]))
