@@ -5,6 +5,7 @@ pub enum Event {
     Copy,
     Escape,
     LeftClick,
+    UpdatePrimaryClipboard,
 }
 
 pub fn events() -> Subscription<(window::Id, Event)> {
@@ -31,6 +32,11 @@ fn filtered_events(
         iced::Event::Mouse(mouse::Event::ButtonPressed(
             mouse::Button::Left,
         )) if ignored(status) => Some(Event::LeftClick),
+        iced::Event::Mouse(mouse::Event::ButtonReleased(
+            mouse::Button::Left,
+        )) if cfg!(target_os = "linux") && ignored(status) => {
+            Some(Event::UpdatePrimaryClipboard)
+        }
         _ => None,
     };
 
