@@ -10,6 +10,7 @@ mod icon;
 mod logger;
 mod modal;
 mod notification;
+mod platform_specific;
 mod screen;
 mod stream;
 mod url;
@@ -35,7 +36,7 @@ use data::{
 };
 use iced::widget::{column, container};
 use iced::{Length, Subscription, Task, padding};
-use screen::{dashboard, help, migration, welcome};
+use screen::{dashboard, help, welcome};
 use tokio::runtime;
 use tokio_stream::wrappers::ReceiverStream;
 
@@ -64,19 +65,22 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     let logs_config = Config::load_logs().unwrap_or_default();
 
+    // spin up a single-threaded tokio runtime to run the logs deletion and
+    // config loading tasks to completion we don't want to wrap our whole
+    // program with a runtime since iced starts its own.
+    let rt = runtime::Builder::new_current_thread()
+        .enable_all()
+        .build()?;
+
+    let _ = rt.block_on(history::delete(&history::Kind::Logs));
+
     let log_stream =
         logger::setup(is_debug, logs_config).expect("setup logging");
     log::info!("halloy {} has started", environment::formatted_version());
     log::info!("config dir: {:?}", environment::config_dir());
     log::info!("data dir: {:?}", environment::data_dir());
 
-    // spin up a single-threaded tokio runtime to run the config loading task to completion
-    // we don't want to wrap our whole program with a runtime since iced starts its own.
     let (config_load, window_load) = {
-        let rt = runtime::Builder::new_current_thread()
-            .enable_all()
-            .build()?;
-
         rt.block_on(async {
             let config = Config::load().await;
             let window = data::Window::load().await;
@@ -84,6 +88,11 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
             (config, window)
         })
     };
+
+    // Futures have only been run via block_on, so we should be able to
+    // shutdown_background without leaks
+    rt.shutdown_background();
+
     // DANGER ZONE - font must be set using config
     // before we do any iced related stuff w/ it
     font::set(config_load.as_ref().ok());
@@ -206,20 +215,8 @@ impl Halloy {
                     command.map(Message::Dashboard),
                 )
             }
-            // If we have a YAML file, but end up in this arm
-            // it means the user tried to load Halloy with a YAML configuration, but it expected TOML.
-            Err(config::Error::ConfigMissing {
-                has_yaml_config: true,
-            }) => (
-                Screen::Migration(screen::Migration::new()),
-                server::Map::default(),
-                Config::default(),
-                Task::none(),
-            ),
             // Show regular welcome screen for new users.
-            Err(config::Error::ConfigMissing {
-                has_yaml_config: false,
-            }) => (
+            Err(config::Error::ConfigMissing) => (
                 Screen::Welcome(screen::Welcome::new()),
                 server::Map::default(),
                 Config::default(),
@@ -258,7 +255,6 @@ pub enum Screen {
     Dashboard(screen::Dashboard),
     Help(screen::Help),
     Welcome(screen::Welcome),
-    Migration(screen::Migration),
     Exit { pending_exit: HashSet<Server> },
 }
 
@@ -270,7 +266,6 @@ pub enum Message {
     Stream(stream::Update),
     Help(help::Message),
     Welcome(welcome::Message),
-    Migration(migration::Message),
     Event(window::Id, Event),
     Tick(Instant),
     Version(Option<String>),
@@ -295,12 +290,15 @@ impl Halloy {
         let position =
             position.map(window::Position::Specific).unwrap_or_default();
 
+        let default_config = Config::default();
+        let config = config_load.as_ref().unwrap_or(&default_config);
+
         let (main_window, open_main_window) = window::open(window::Settings {
             size,
             position,
             min_size: Some(window::MIN_SIZE),
             exit_on_close_request: false,
-            ..window::settings()
+            ..window::settings(config)
         });
 
         let (mut halloy, command) =
@@ -342,6 +340,7 @@ impl Halloy {
                             styles,
                             &self.main_window,
                             &mut self.theme,
+                            &self.config,
                         )
                         .map(Message::Dashboard);
                 }
@@ -388,7 +387,7 @@ impl Halloy {
                 );
 
                 // Retrack after dashboard state changes
-                let track = dashboard.track();
+                let track = dashboard.track(Some(&self.clients));
 
                 let event_task = match event {
                     Some(dashboard::Event::ConfigReloaded(config)) => {
@@ -406,8 +405,7 @@ impl Halloy {
                                     if let Some(server) =
                                         self.servers.get_mut(&server)
                                     {
-                                        Arc::make_mut(server).order =
-                                            config.order;
+                                        *server = config.clone();
                                     } else {
                                         self.servers
                                             .insert(server, config.clone());
@@ -421,6 +419,11 @@ impl Halloy {
                                     .current_mode
                                     .theme(&updated.appearance.selected)
                                     .into();
+
+                                // Load new notification sounds.
+                                self.notifications =
+                                    Notifications::new(&updated);
+
                                 self.config = updated;
 
                                 for (server, _) in removed_servers {
@@ -442,6 +445,7 @@ impl Halloy {
                                 );
                             }
                         };
+
                         Task::none()
                     }
                     Some(dashboard::Event::ReloadThemes) => {
@@ -542,21 +546,6 @@ impl Halloy {
                     None => Task::none(),
                 }
             }
-            Message::Migration(message) => {
-                let Screen::Migration(migration) = &mut self.screen else {
-                    return Task::none();
-                };
-
-                match migration.update(message) {
-                    Some(migration::Event::RefreshConfiguration) => {
-                        Task::perform(
-                            Config::load(),
-                            Message::ScreenConfigReloaded,
-                        )
-                    }
-                    None => Task::none(),
-                }
-            }
             Message::Stream(update) => match update {
                 stream::Update::Disconnected {
                     server,
@@ -1096,6 +1085,10 @@ impl Halloy {
                                         );
                                     }
                                     data::client::Event::AddedIsupportParam(param) => {
+                                        if matches!(param, data::isupport::Parameter::CASEMAPPING(_)) {
+                                            dashboard.renormalize_history(&server, &self.clients);
+                                        }
+
                                         if matches!(
                                             param,
                                             data::isupport::Parameter::CASEMAPPING(_)
@@ -1104,12 +1097,14 @@ impl Halloy {
                                             let chantypes = self.clients.get_chantypes(&server);
                                             let casemapping = self.clients.get_casemapping(&server);
 
-                                            FilterChain::sync_channels(
+                                            FilterChain::sync_isupport(
                                                 dashboard.get_filters(),
                                                 &server,
                                                 chantypes,
                                                 casemapping
                                             );
+
+                                            dashboard.reprocess_history(&self.clients, &self.config.buffer);
                                         }
                                     }
                                     Event::BouncerNetwork(server, config) => {
@@ -1190,7 +1185,7 @@ impl Halloy {
                 }
 
                 if let Screen::Dashboard(dashboard) = &mut self.screen {
-                    dashboard.tick(now).map(Message::Dashboard)
+                    dashboard.tick(now, &self.clients).map(Message::Dashboard)
                 } else {
                     Task::none()
                 }
@@ -1420,10 +1415,8 @@ impl Halloy {
     }
 
     fn view(&self, id: window::Id) -> Element<'_, Message> {
-        // The height margin varies across different operating systems due to design differences.
-        // For instance, on macOS, the menubar is hidden, resulting in a need for additional padding to accommodate the
-        // space occupied by the traffic light buttons.
-        let height_margin = if cfg!(target_os = "macos") { 20 } else { 0 };
+        let platform_specific_padding =
+            platform_specific::content_padding(&self.config);
 
         // Main window.
         if id == self.main_window.id {
@@ -1441,9 +1434,6 @@ impl Halloy {
                 Screen::Welcome(welcome) => {
                     welcome.view(&self.theme).map(Message::Welcome)
                 }
-                Screen::Migration(migration) => {
-                    migration.view().map(Message::Migration)
-                }
                 Screen::Exit { .. } => column![].into(),
             };
 
@@ -1453,7 +1443,7 @@ impl Halloy {
                     .height(Length::Fill)
                     .style(theme::container::general),
             )
-            .padding(padding::top(height_margin));
+            .padding(padding::top(platform_specific_padding));
 
             // Modals might have a id representing which window to be presented on.
             // If modal has no id, we show them on main_window.
@@ -1477,7 +1467,7 @@ impl Halloy {
                     .view_window(id, &self.clients, &self.config, &self.theme)
                     .map(Message::Dashboard),
             )
-            .padding(padding::top(height_margin));
+            .padding(padding::top(platform_specific_padding));
 
             // Modals might have a id representing which window to be presented on.
             // If modal id match the current id we show it.
