@@ -1,9 +1,7 @@
 pub mod dashboard;
 pub mod help;
-pub mod migration;
 pub mod welcome;
 
 pub use dashboard::Dashboard;
 pub use help::Help;
-pub use migration::Migration;
 pub use welcome::Welcome;
