@@ -221,32 +221,36 @@ impl TitleBar {
                 None
             },
             {
-                let can_scroll_to_bottom =
-                    !buffer.is_scrolled_to_bottom().unwrap_or_default();
-                let scroll_to_bottom_button =
-                    button(center(icon::scroll_to_bottom()))
-                        .padding(5)
-                        .width(22)
-                        .height(22)
-                        .on_press_maybe(
-                            can_scroll_to_bottom
-                                .then_some(Message::ScrollToBottom),
-                        )
-                        .style(|theme, status| {
-                            theme::button::secondary(theme, status, false)
-                        });
+                if maybe_buffer_kind.is_some() {
+                    let can_scroll_to_bottom =
+                        !buffer.is_scrolled_to_bottom().unwrap_or_default();
+                    let scroll_to_bottom_button =
+                        button(center(icon::scroll_to_bottom()))
+                            .padding(5)
+                            .width(22)
+                            .height(22)
+                            .on_press_maybe(
+                                can_scroll_to_bottom
+                                    .then_some(Message::ScrollToBottom),
+                            )
+                            .style(|theme, status| {
+                                theme::button::secondary(theme, status, false)
+                            });
 
-                let scroll_to_bottom_button_with_tooltip = tooltip(
-                    scroll_to_bottom_button,
-                    show_tooltips.then_some(if can_scroll_to_bottom {
-                        "Scroll to bottom"
-                    } else {
-                        "Already at bottom"
-                    }),
-                    tooltip::Position::Bottom,
-                    theme,
-                );
-                Some(scroll_to_bottom_button_with_tooltip)
+                    let scroll_to_bottom_button_with_tooltip = tooltip(
+                        scroll_to_bottom_button,
+                        show_tooltips.then_some(if can_scroll_to_bottom {
+                            "Scroll to bottom"
+                        } else {
+                            "Already at bottom"
+                        }),
+                        tooltip::Position::Bottom,
+                        theme,
+                    );
+                    Some(scroll_to_bottom_button_with_tooltip)
+                } else {
+                    None
+                }
             },
             if let Buffer::Channel(state) = &buffer {
                 if let Some(topic) =
@@ -254,8 +258,8 @@ impl TitleBar {
                     && topic.content.is_some()
                 {
                     let topic_enabled = settings.map_or(
-                        config.buffer.channel.topic.enabled,
-                        |settings| settings.channel.topic.enabled,
+                        config.buffer.channel.topic_banner.enabled,
+                        |settings| settings.channel.topic_banner.enabled,
                     );
 
                     let topic_button = button(center(icon::topic()))
@@ -409,8 +413,8 @@ impl TitleBar {
                 )
                 .shaping(text::Shaping::Advanced),
         )
-        .height(22)
-        .padding([0, 10])
+        .height(theme::line_height(&config.font).ceil().max(22.0))
+        .padding([0, 4])
         .align_y(iced::alignment::Vertical::Center);
 
         widget::TitleBar::new(title)
