@@ -4,15 +4,16 @@ use data::config::{self, Config, sidebar};
 use data::dashboard::{BufferAction, BufferFocusedAction};
 use data::{Version, buffer, file_transfer, history, server};
 use iced::widget::{
-    Column, Row, Scrollable, Space, button, column, container, horizontal_rule,
-    pane_grid, row, scrollable, stack, text, vertical_rule, vertical_space,
+    Column, Row, Scrollable, Space, button, column, container, pane_grid, row,
+    rule, scrollable, space, stack, text,
 };
 use iced::{Alignment, Length, Padding, Task, padding};
+use itertools::Either;
 use tokio::time;
 
 use super::{Focus, Panes, Server};
 use crate::widget::{Element, Text, context_menu, double_pass};
-use crate::{Theme, font, icon, theme, window};
+use crate::{Theme, font, icon, platform_specific, theme, window};
 
 const CONFIG_RELOAD_DELAY: Duration = Duration::from_secs(1);
 
@@ -24,6 +25,7 @@ pub enum Message {
     Replace(buffer::Upstream),
     Close(window::Id, pane_grid::Pane),
     Swap(window::Id, pane_grid::Pane),
+    Detach(buffer::Upstream),
     Leave(buffer::Upstream),
     ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
@@ -46,6 +48,7 @@ pub enum Event {
     Replace(buffer::Upstream),
     Close(window::Id, pane_grid::Pane),
     Swap(window::Id, pane_grid::Pane),
+    Detach(buffer::Upstream),
     Leave(buffer::Upstream),
     ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
@@ -103,6 +106,9 @@ impl Sidebar {
             Message::Swap(window, pane) => {
                 (Task::none(), Some(Event::Swap(window, pane)))
             }
+            Message::Detach(buffer) => {
+                (Task::none(), Some(Event::Detach(buffer)))
+            }
             Message::Leave(buffer) => {
                 (Task::none(), Some(Event::Leave(buffer)))
             }
@@ -157,7 +163,7 @@ impl Sidebar {
     ) -> Element<'a, Message> {
         let base = button(icon::menu()).padding(5).width(Length::Shrink);
 
-        let menu = Menu::list();
+        let menu = Menu::list(version.is_old());
 
         let logs_has_unread = history.has_unread(&history::Kind::Logs);
 
@@ -285,38 +291,34 @@ impl Sidebar {
                                 Message::ToggleThemeEditor,
                             ),
                             Menu::HorizontalRule => match length {
-                                Length::Fill => container(horizontal_rule(1))
+                                Length::Fill => container(rule::horizontal(1))
                                     .padding([0, 6])
                                     .into(),
-                                _ => Space::new(length, 1).into(),
+                                _ => {
+                                    Space::new().width(length).height(1).into()
+                                }
                             },
-                            Menu::Version => match version.is_old() {
-                                true => context_button(
-                                    text("New version available")
-                                        .style(theme::text::tertiary)
-                                        .font_maybe(
-                                            theme::font_style::tertiary(theme)
-                                                .map(font::get),
-                                        ),
-                                    None,
-                                    icon::megaphone()
-                                        .style(theme::text::tertiary),
-                                    Message::OpenReleaseWebsite,
-                                ),
-                                false => container(
-                                    text(format!(
-                                        "Halloy ({})",
-                                        version.current
-                                    ))
+                            Menu::Update => context_button(
+                                text("New version available")
+                                    .style(theme::text::tertiary)
+                                    .font_maybe(
+                                        theme::font_style::tertiary(theme)
+                                            .map(font::get),
+                                    ),
+                                None,
+                                icon::megaphone().style(theme::text::tertiary),
+                                Message::OpenReleaseWebsite,
+                            ),
+                            Menu::Version => container(
+                                text(format!("Halloy ({})", version.current))
                                     .style(theme::text::secondary)
                                     .font_maybe(
                                         theme::font_style::secondary(theme)
                                             .map(font::get),
                                     ),
-                                )
-                                .padding(5)
-                                .into(),
-                            },
+                            )
+                            .padding(5)
+                            .into(),
                             Menu::Documentation => context_button(
                                 text("Documentation"),
                                 None,
@@ -382,6 +384,7 @@ impl Sidebar {
                     |buffer: buffer::Upstream,
                      connected: bool,
                      server_has_unread: bool,
+                     supports_detach: bool,
                      has_unread: bool,
                      has_highlight: bool| {
                         upstream_buffer_button(
@@ -395,6 +398,7 @@ impl Sidebar {
                             config.sidebar.server_icon_size,
                             config.sidebar.unread_indicator,
                             server_has_unread,
+                            supports_detach,
                             has_unread,
                             has_highlight,
                             width,
@@ -412,6 +416,7 @@ impl Sidebar {
                                 buffer::Upstream::Server(server.clone()),
                                 false,
                                 history.server_has_unread(server.clone()),
+                                clients.get_server_supports_detach(server),
                                 history.has_unread(&history::Kind::Server(
                                     server.clone(),
                                 )),
@@ -426,6 +431,7 @@ impl Sidebar {
                                 buffer::Upstream::Server(server.clone()),
                                 true,
                                 history.server_has_unread(server.clone()),
+                                clients.get_server_supports_detach(server),
                                 history.has_unread(&history::Kind::Server(
                                     server.clone(),
                                 )),
@@ -443,6 +449,7 @@ impl Sidebar {
                                     ),
                                     true,
                                     history.server_has_unread(server.clone()),
+                                    clients.get_server_supports_detach(server),
                                     history.has_unread(
                                         &history::Kind::Channel(
                                             server.clone(),
@@ -472,6 +479,7 @@ impl Sidebar {
                                     ),
                                     true,
                                     history.server_has_unread(server.clone()),
+                                    clients.get_server_supports_detach(server),
                                     history.has_unread(&history::Kind::Query(
                                         server.clone(),
                                         query.clone(),
@@ -489,7 +497,7 @@ impl Sidebar {
                             if config.sidebar.position.is_horizontal() {
                                 if client_enumeration < clients.len() {
                                     buffers.push(
-                                        container(vertical_rule(1))
+                                        container(rule::vertical(1))
                                             .padding(padding::top(6))
                                             .height(20)
                                             .width(12)
@@ -499,7 +507,7 @@ impl Sidebar {
                                 }
                             } else {
                                 buffers
-                                    .push(vertical_space().height(12).into());
+                                    .push(space::vertical().height(12).into());
                             }
                         }
                     }
@@ -557,18 +565,33 @@ impl Sidebar {
             }
         };
 
+        let platform_specific_padding =
+            platform_specific::sidebar_padding(config);
+
         let padding = match config.sidebar.position {
-            sidebar::Position::Left => padding::top(8).bottom(6).left(6),
-            sidebar::Position::Right => padding::top(8).bottom(6).right(6),
-            sidebar::Position::Top => padding::top(8).left(6).right(6),
-            sidebar::Position::Bottom => padding::bottom(8).left(6).right(6),
+            sidebar::Position::Left => {
+                padding::top(8 + platform_specific_padding)
+                    .bottom(6)
+                    .left(6)
+            }
+            sidebar::Position::Right => {
+                padding::top(8 + platform_specific_padding)
+                    .bottom(6)
+                    .right(6)
+            }
+            sidebar::Position::Top => {
+                padding::top(8 + platform_specific_padding).right(6)
+            }
+            sidebar::Position::Bottom => padding::bottom(8)
+                .left(6)
+                .right(6)
+                .top(platform_specific_padding),
         };
 
         let content = if config.sidebar.position.is_horizontal() {
             container(
                 content(Length::Shrink).width(Length::Fill).padding(padding),
             )
-            .into()
         } else {
             let first_pass = content(Length::Shrink);
             let second_pass = content(Length::Fill);
@@ -579,10 +602,9 @@ impl Sidebar {
                 )
                 .width(Length::Shrink)
                 .padding(padding)
-                .into()
         };
 
-        Some(content)
+        Some(content.into())
     }
 }
 
@@ -595,25 +617,33 @@ enum Menu {
     Logs,
     FileTransfers,
     Version,
+    Update,
     HorizontalRule,
     Documentation,
     OpenConfigFile,
 }
 
 impl Menu {
-    fn list() -> Vec<Self> {
-        vec![
-            Menu::Version,
-            Menu::HorizontalRule,
-            Menu::CommandBar,
-            Menu::Documentation,
-            Menu::FileTransfers,
-            Menu::Highlights,
-            Menu::Logs,
-            Menu::OpenConfigFile,
-            Menu::RefreshConfig,
-            Menu::ThemeEditor,
-        ]
+    fn list(has_new_version: bool) -> Vec<Self> {
+        let mut list = vec![Self::Version];
+
+        if has_new_version {
+            list.push(Self::Update);
+        }
+
+        list.extend([
+            Self::HorizontalRule,
+            Self::CommandBar,
+            Self::Documentation,
+            Self::FileTransfers,
+            Self::Highlights,
+            Self::Logs,
+            Self::OpenConfigFile,
+            Self::RefreshConfig,
+            Self::ThemeEditor,
+        ]);
+
+        list
     }
 }
 
@@ -627,6 +657,7 @@ enum Entry {
     Close(window::Id, pane_grid::Pane),
     Swap(window::Id, pane_grid::Pane),
     Leave,
+    Detach,
 }
 
 impl Entry {
@@ -635,6 +666,7 @@ impl Entry {
         num_panes: usize,
         open: Option<(window::Id, pane_grid::Pane)>,
         focus: Focus,
+        supports_detach: bool,
     ) -> Vec<Self> {
         [
             match buffer {
@@ -645,23 +677,32 @@ impl Entry {
                 buffer::Upstream::Query(_, _) => vec![],
             },
             match open {
-                None => vec![
-                    Entry::MarkAsRead,
-                    Entry::NewPane,
-                    Entry::Popout,
-                    Entry::Replace,
-                    Entry::Leave,
-                ],
-                Some((window, pane)) => (num_panes > 1)
-                    .then_some(Entry::Close(window, pane))
+                None => Either::Left(
+                    vec![
+                        Entry::MarkAsRead,
+                        Entry::NewPane,
+                        Entry::Popout,
+                        Entry::Replace,
+                    ]
                     .into_iter()
                     .chain(
-                        (Focus { window, pane } != focus)
-                            .then_some(Entry::Swap(window, pane)),
-                    )
-                    .chain(Some(Entry::Leave))
-                    .collect(),
-            },
+                        (matches!(buffer, buffer::Upstream::Channel(_, _))
+                            && supports_detach)
+                            .then_some(Entry::Detach),
+                    ),
+                ),
+                Some((window, pane)) => Either::Right(
+                    (num_panes > 1)
+                        .then_some(Entry::Close(window, pane))
+                        .into_iter()
+                        .chain(
+                            (Focus { window, pane } != focus)
+                                .then_some(Entry::Swap(window, pane)),
+                        ),
+                ),
+            }
+            .chain(Some(Entry::Leave))
+            .collect(),
         ]
         .concat()
     }
@@ -678,6 +719,7 @@ fn upstream_buffer_button<'a>(
     server_icon_size: u32,
     unread_indicator: sidebar::UnreadIndicator,
     server_has_unread: bool,
+    supports_detach: bool,
     has_unread: bool,
     has_highlight: bool,
     width: Length,
@@ -757,7 +799,7 @@ fn upstream_buffer_button<'a>(
 
     let (left_padding, icon) = if position.is_horizontal() {
         icon_tuple.map_or((0, None), |(icon, icon_size)| {
-            (icon_size + 8, Some(container(icon).center_y(17.0)))
+            (icon_size + 8, Some(container(icon).center_y(Length::Fill)))
         })
     } else {
         let max_icon_size = server_icon_size.max(
@@ -774,7 +816,9 @@ fn upstream_buffer_button<'a>(
         (
             max_icon_size + 8,
             icon_tuple.map(|(icon, _)| {
-                container(icon).center_y(17.0).center_x(max_icon_size)
+                container(icon)
+                    .center_y(Length::Fill)
+                    .center_x(max_icon_size)
             }),
         )
     };
@@ -788,7 +832,7 @@ fn upstream_buffer_button<'a>(
                             .style(buffer_title_style)
                             .font_maybe(buffer_title_font.clone())
                             .shaping(text::Shaping::Advanced),
-                        Space::with_width(6),
+                        Space::new().width(6),
                         text(server.name.to_string())
                             .style(theme::text::secondary)
                             .font_maybe(buffer_title_font)
@@ -818,51 +862,53 @@ fn upstream_buffer_button<'a>(
         icon
     ]);
 
-    let base = button(content.height(17.0).width(width))
-        .style(move |theme, status| {
-            theme::button::sidebar_buffer(
-                theme,
-                status,
-                is_focused.is_some(),
-                open.is_some(),
-            )
-        })
-        .on_press({
-            match is_focused {
-                Some((window, pane)) => {
-                    if let Some(focus_action) = focused_buffer_action {
-                        match focus_action {
-                            BufferFocusedAction::ClosePane => {
-                                Message::Close(window, pane)
+    let base =
+        button(content.width(width).padding(Padding::default().bottom(1)))
+            .style(move |theme, status| {
+                theme::button::sidebar_buffer(
+                    theme,
+                    status,
+                    is_focused.is_some(),
+                    open.is_some(),
+                )
+            })
+            .on_press({
+                match is_focused {
+                    Some((window, pane)) => {
+                        if let Some(focus_action) = focused_buffer_action {
+                            match focus_action {
+                                BufferFocusedAction::ClosePane => {
+                                    Message::Close(window, pane)
+                                }
                             }
+                        } else {
+                            // Re-focus pane on press instead of disabling the button in order
+                            // to have hover status of the button for styling
+                            Message::Focus(window, pane)
                         }
-                    } else {
-                        // Re-focus pane on press instead of disabling the button in order
-                        // to have hover status of the button for styling
-                        Message::Focus(window, pane)
                     }
-                }
-                None => {
-                    if let Some((window, pane)) = open {
-                        Message::Focus(window, pane)
-                    } else {
-                        match buffer_action {
-                            BufferAction::NewPane => {
-                                Message::New(buffer.clone())
-                            }
-                            BufferAction::ReplacePane => {
-                                Message::Replace(buffer.clone())
-                            }
-                            BufferAction::NewWindow => {
-                                Message::Popout(buffer.clone())
+                    None => {
+                        if let Some((window, pane)) = open {
+                            Message::Focus(window, pane)
+                        } else {
+                            match buffer_action {
+                                BufferAction::NewPane => {
+                                    Message::New(buffer.clone())
+                                }
+                                BufferAction::ReplacePane => {
+                                    Message::Replace(buffer.clone())
+                                }
+                                BufferAction::NewWindow => {
+                                    Message::Popout(buffer.clone())
+                                }
                             }
                         }
                     }
                 }
-            }
-        });
+            });
 
-    let entries = Entry::list(&buffer, panes.len(), open, focus);
+    let entries =
+        Entry::list(&buffer, panes.len(), open, focus, supports_detach);
 
     if entries.is_empty() || !connected {
         base.into()
@@ -915,6 +961,10 @@ fn upstream_buffer_button<'a>(
                         "Swap with current pane",
                         Some(Message::Swap(window, pane)),
                     ),
+                    Entry::Detach => (
+                        "Detach from channel",
+                        Some(Message::Detach(buffer.clone())),
+                    ),
                     Entry::Leave => (
                         match &buffer {
                             buffer::Upstream::Server(_) => "Leave server",
