@@ -31,11 +31,13 @@ where
                     modifiers: m,
                     ..
                 }) = &event
-                    && key == *k && modifiers == *m {
-                        shell.publish(on_press.clone());
-                        shell.capture_event();
-                        return;
-                    }
+                    && key == *k
+                    && modifiers == *m
+                {
+                    shell.publish(on_press.clone());
+                    shell.capture_event();
+                    return;
+                }
 
                 inner.as_widget_mut().update(
                     tree, event, layout, cursor, renderer, clipboard, shell,
