@@ -218,9 +218,9 @@ where
             | iced::Event::Touch(touch::Event::FingerMoved { .. }) => {
                 if let Some(cursor) = cursor.position()
                     && let Interaction::Selecting(raw) = &mut state.interaction
-                    {
-                        raw.end = cursor;
-                    }
+                {
+                    raw.end = cursor;
+                }
             }
             _ => {}
         }
@@ -502,13 +502,10 @@ pub fn selected<Message: Send + 'static>(
     }
 
     impl<T> Operation<T> for Selected<T> {
-        fn container(
-            &mut self,
-            _id: Option<&widget::Id>,
-            _bounds: Rectangle,
-            operate_on_children: &mut dyn FnMut(&mut dyn Operation<T>),
-        ) {
-            operate_on_children(self);
+        fn container(&mut self, _id: Option<&widget::Id>, _bounds: Rectangle) {}
+
+        fn traverse(&mut self, operate: &mut dyn FnMut(&mut dyn Operation<T>)) {
+            operate(self);
         }
 
         fn custom(
